package bogqaai;

import bogqaai.graph.*;
import bogqaai.graph.algorithm.*;

/**
 * A szerkesztőablakban megjelenő algoritmus vezérlő felület. Grafikus,
 * <i>javax.swing.JPanel</i> leszármazott osztály, a NetBeans szerkesztőjével
 * készült. Az algoritmusok léptetését és animálását <i>javax.swing.Timer</i>
 * segítségével valósítja meg, ehhez több belső állandót és metódust használ.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class AlgorithmPanel extends javax.swing.JPanel {
    /**
     * Az adatszerkezetet kirajzoló belső osztály. Az absztrakt gráfot kirajzoló
     * <i>DrawerPanel</i> osztályból származik.
     */
    private class DataDrawerPanel extends DrawerPanel<AbstractData> {
        /** Az adatszerkezet. */
        private final AbstractData data;

        /**
         * A rajzoló osztály konstruktora, paramétere az adatszerkezet.
         *
         * @param data A kirajzolandó adatszerkezet
         */
        public DataDrawerPanel(AbstractData data) {
            super(data);
            this.data = data;
            setPreferredSize(new java.awt.Dimension(data.getDrawerWidth(), data.getDrawerHeight()));
        }

        /**
         * Adatszerkezet kirajzolása.
         *
         * @param g A rajzolási cél
         */
        @Override
        protected void paintGraphics(java.awt.Graphics2D g){
            super.paintGraphics(g);
            data.draw(g);
        }
    }

    /** Késleltetés értékek. */
    private static final int[] DELAYS = {
        100, 200, 500, 1000, 2000, 5000, 10000
    };
    /** Alapértelmezett lépésszám. */
    private static final int DEFAULT_COUNT = 12;
    /** Késleltetésekhez tartozó lépésszám. */
    private static final int[] COUNTS = {
        0, 2, 6, DEFAULT_COUNT, DEFAULT_COUNT, DEFAULT_COUNT, DEFAULT_COUNT
    };

    /** Koordináló objektum. */
    private final MainWindow parent;
    /** Gráfot tartalmazó objektum. */
    private final Graph graph;
    /** Adatszerkezetet kirajzoló felület. */
    private DataDrawerPanel dataPanel = null;
    /** Időzítéshez használt <i>Timer</i>. */
    private javax.swing.Timer timer = null;
    /** Lépés animálásához használt <i>Timer</i>. */
    private javax.swing.Timer algTimer = null;
    /** Lépés animálás aktuális állapota. */
    private int algTimerT = 1;
    /** Lépés animáláskor a megjelenített állapotok száma. */
    private int algTimerMax = 1;

    /**
     * Az algoritmus vezérlő felület konstruktora, paramétere a koordináló objektum
     * és a gráf.
     *
     * @param parent Koordináló objektum
     * @param graph Gráf
     */
    public AlgorithmPanel(MainWindow parent, Graph graph) {
        initComponents();
        this.parent = parent;
        this.graph = graph;
    }

    /**
     * Vezérlők letiltása és engedélyezése lejátszás közben.
     *
     * @param enable Ha értéke igaz, akkor engedélyezés, különben letiltás
     */
    private void enablePlayer(boolean enable){
        playButton.setEnabled(enable);
        nextButton.setEnabled(enable);
        stopButton.setEnabled(!enable);
        delayEdit.setEnabled(enable);
    }
    /**
     * Az összes vezérlő letiltása algoritmus szemléltés végén.
     */
    private void disableAll(){
        playButton.setEnabled(false);
        nextButton.setEnabled(false);
        stopButton.setEnabled(false);
        delayEdit.setEnabled(false);
    }

    /**
     * Időzítés és lépés animálás megállítása.
     */
    private void stopAll(){
        stopTimer();
        stopAlgTimer();
        parent.setStatus("");
    }
    /**
     * Időzítés megállítása.
     */
    private void stopTimer(){
        if(timer != null){
            timer.stop();
            timer = null;
        }
    }
    /**
     * Lépés animálás megállítása.
     */
    private void stopAlgTimer(){
        if(algTimer != null){
            algTimer.stop();
            algTimer = null;
            Algorithm alg = parent.getAlgorithm();
            if(alg != null){
                alg.changed();
            }else{
                graph.changed();
            }
        }
    }
    /**
     * Időzítés indítása.
     *
     * @param id Késleltetés értékek sorszáma
     */
    private void startTimer(final int id){
        timer = new javax.swing.Timer(DELAYS[id], new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                if(evt.getSource() == timer){
                    Algorithm alg = parent.getAlgorithm();
                    if(alg != null){
                        stopAlgTimer();
                        alg.step();
                        startAlgTimer(COUNTS[id]);
                        if(alg.isReady()){
                            stopTimer();
                            disableAll();
                        }
                        parent.setStatus(alg.getText());
                    }else{
                        stopAll();
                    }
                }
            }
        });
        timer.setRepeats(true);
        timer.setInitialDelay(100);
        timer.start();
    }
    /**
     * Lépés animálás indítása.
     *
     * @param max A megjelenítendő állapotok száma
     */
    private void startAlgTimer(int max){
        if(max > 0){
            algTimerT = 0;
            algTimerMax = max;
            algTimer = new javax.swing.Timer(40, new java.awt.event.ActionListener() {
                @Override
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    if(evt.getSource() == algTimer){
                        Algorithm alg = parent.getAlgorithm();
                        if(alg != null && algTimerT < algTimerMax){
                            alg.animation((float)algTimerT/algTimerMax);
                            ++algTimerT;
                        }else{
                            stopAlgTimer();
                        }
                    }
                }
            });
            algTimer.setRepeats(true);
            algTimer.setInitialDelay(0);
            algTimer.start();
        }else{
            Algorithm alg = parent.getAlgorithm();
            if(alg != null){
                alg.changed();
            }else{
                graph.changed();
            }
        }
    }

    /**
     * Algoritmus szemléltetés indítása, paramétere az algoritmus adatait leíró
     * rekord.
     *
     * @param strings Az algoritmus adatait leíró rekord
     */
    public void start(Algorithm.Strings strings){
        stopAll();
        titleLabel.setText(strings.title);
        descriptionLabel.setText(strings.description);
        dataLabel.setText(strings.data);
        Algorithm alg = parent.getAlgorithm();
        if(alg != null){
            AbstractData data = alg.getData();
            dataPanel = data != null ? new DataDrawerPanel(data) : null;
            dataPane.setViewportView(dataPanel);
            alg.reset();
            if(alg.isReady()){
                disableAll();
            }else{
                enablePlayer(true);
            }
            parent.setStatus(alg.getText());
        }
    }
    /**
     * Algoritmus szemléltetés leállítása.
     */
    public void stop(){
        stopAll();
        disableAll();
        titleLabel.setText(" ");
        descriptionLabel.setText(" ");
        dataLabel.setText(" ");
        dataPanel = null;
        dataPane.setViewportView(null);
    }

    /**
     * Az adatszerkezetet kirajzoló felület lekérdezése.
     *
     * @return Az adatszerkezetet kirajzoló felület
     */
    public DrawerPanel getDataPanel(){
        return dataPanel;
    }

    /**
     * A grafikus komponensek inicializálása a konstruktorban. Tartalmát a
     * NetBeans szerkesztője generálta.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.JToolBar titleBar = new javax.swing.JToolBar();
        javax.swing.JPanel titlePanel = new javax.swing.JPanel();
        titleLabel = new javax.swing.JLabel();
        javax.swing.JButton closeButton = new javax.swing.JButton();
        javax.swing.JScrollPane descriptionLabelPane = new javax.swing.JScrollPane();
        descriptionLabel = new javax.swing.JLabel();
        resetButton = new javax.swing.JButton();
        resultButton = new javax.swing.JButton();
        playButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        nextButton = new javax.swing.JButton();
        delayEdit = new javax.swing.JComboBox();
        javax.swing.JScrollPane dataLabelPane = new javax.swing.JScrollPane();
        dataLabel = new javax.swing.JLabel();
        dataPane = new javax.swing.JScrollPane();

        setPreferredSize(new java.awt.Dimension(240, 300));

        titleBar.setFloatable(false);
        titleBar.setRollover(true);

        titlePanel.setBackground(new java.awt.Color(255, 255, 255, 0));

        titleLabel.setFont(titleLabel.getFont().deriveFont((float)18));
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Bundle"); // NOI18N
        titleLabel.setText(bundle.getString("AlgorithmPanel.titleLabel.text")); // NOI18N

        javax.swing.GroupLayout titlePanelLayout = new javax.swing.GroupLayout(titlePanel);
        titlePanel.setLayout(titlePanelLayout);
        titlePanelLayout.setHorizontalGroup(
            titlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(titlePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(titleLabel)
                .addContainerGap(121, Short.MAX_VALUE))
        );
        titlePanelLayout.setVerticalGroup(
            titlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(titlePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(titleLabel)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        titleBar.add(titlePanel);

        closeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/close.png"))); // NOI18N
        closeButton.setToolTipText(bundle.getString("AlgorithmPanel.closeButton.toolTipText")); // NOI18N
        closeButton.setContentAreaFilled(false);
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });
        titleBar.add(closeButton);

        descriptionLabelPane.setBorder(null);
        descriptionLabelPane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        descriptionLabelPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

        descriptionLabel.setText(bundle.getString("AlgorithmPanel.descriptionLabel.text")); // NOI18N
        descriptionLabelPane.setViewportView(descriptionLabel);

        resetButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/reset.png"))); // NOI18N
        resetButton.setText(bundle.getString("AlgorithmPanel.resetButton.text")); // NOI18N
        resetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetButtonActionPerformed(evt);
            }
        });

        resultButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/result.png"))); // NOI18N
        resultButton.setText(bundle.getString("AlgorithmPanel.resultButton.text")); // NOI18N
        resultButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resultButtonActionPerformed(evt);
            }
        });

        playButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/play.png"))); // NOI18N
        playButton.setToolTipText(bundle.getString("AlgorithmPanel.playButton.toolTipText")); // NOI18N
        playButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        playButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playButtonActionPerformed(evt);
            }
        });

        stopButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/stop.png"))); // NOI18N
        stopButton.setToolTipText(bundle.getString("AlgorithmPanel.stopButton.toolTipText")); // NOI18N
        stopButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        nextButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/next.png"))); // NOI18N
        nextButton.setToolTipText(bundle.getString("AlgorithmPanel.nextButton.toolTipText")); // NOI18N
        nextButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        nextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextButtonActionPerformed(evt);
            }
        });

        delayEdit.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "100 ms", "200 ms", "500 ms", "1 s", "2 s", "5 s", "10 s" }));
        delayEdit.setSelectedIndex(3);
        delayEdit.setToolTipText(bundle.getString("AlgorithmPanel.delayEdit.toolTipText")); // NOI18N

        dataLabelPane.setBorder(null);
        dataLabelPane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        dataLabelPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

        dataLabel.setText(bundle.getString("AlgorithmPanel.dataLabel.text")); // NOI18N
        dataLabelPane.setViewportView(dataLabel);

        dataPane.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(titleBar, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(dataLabelPane)
                    .addComponent(descriptionLabelPane)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(playButton)
                        .addGap(3, 3, 3)
                        .addComponent(stopButton)
                        .addGap(9, 9, 9)
                        .addComponent(nextButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(delayEdit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(resetButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGap(10, 10, 10)
                        .addComponent(resultButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addComponent(dataPane))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(titleBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(descriptionLabelPane, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(resetButton)
                    .addComponent(resultButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(playButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(stopButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(nextButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(delayEdit, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(dataLabelPane, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(dataPane, javax.swing.GroupLayout.DEFAULT_SIZE, 120, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Bezárás gomb eseménye.
     */
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        parent.selectNone();
    }//GEN-LAST:event_closeButtonActionPerformed

    /**
     * Alaphelyzet gomb eseménye.
     */
    private void resetButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetButtonActionPerformed
        stopAll();
        Algorithm alg = parent.getAlgorithm();
        if(alg != null){
            alg.reset();
            if(alg.isReady()){
                disableAll();
            }else{
                enablePlayer(true);
            }
            parent.setStatus(alg.getText());
        }
    }//GEN-LAST:event_resetButtonActionPerformed

    /**
     * Algoritmus léptetés gomb eseménye.
     */
    private void nextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextButtonActionPerformed
        stopAll();
        Algorithm alg = parent.getAlgorithm();
        if(alg != null && !alg.isReady()){
            stopAlgTimer();
            alg.step();
            startAlgTimer(DEFAULT_COUNT);
            if(alg.isReady()){
                disableAll();
            }
            parent.setStatus(alg.getText());
        }
    }//GEN-LAST:event_nextButtonActionPerformed

    /**
     * Az eredmény gomb eseménye.
     */
    private void resultButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resultButtonActionPerformed
        stopAll();
        disableAll();
        Algorithm alg = parent.getAlgorithm();
        if(alg != null){
            alg.run();
            parent.setStatus(alg.getText());
        }
    }//GEN-LAST:event_resultButtonActionPerformed

    /**
     * Az időzített lejátszás gomb eseménye.
     */
    private void playButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playButtonActionPerformed
        stopAll();
        enablePlayer(false);
        startTimer(delayEdit.getSelectedIndex());
    }//GEN-LAST:event_playButtonActionPerformed

    /**
     * Az időzített lejátszás megállítás gomb eseménye.
     */
    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        stopAll();
        Algorithm alg = parent.getAlgorithm();
        if(alg != null){
            if(alg.isReady()){
                disableAll();
            }else{
                enablePlayer(true);
            }
            parent.setStatus(alg.getText());
        }
    }//GEN-LAST:event_stopButtonActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    /** Adatszerkezet címkéje. */
    private javax.swing.JLabel dataLabel;
    /** Adatszerkezetet kirajzoló felület befoglalója. */
    private javax.swing.JScrollPane dataPane;
    /** Késleltetések listája. */
    private javax.swing.JComboBox delayEdit;
    /** Algoritmus leírás címkéje. */
    private javax.swing.JLabel descriptionLabel;
    /** Algoritmus léptetés gomb. */
    private javax.swing.JButton nextButton;
    /** Időzített lejátszás gomb. */
    private javax.swing.JButton playButton;
    /** Alaphelyzet gomb. */
    private javax.swing.JButton resetButton;
    /** Eredmény gomb. */
    private javax.swing.JButton resultButton;
    /** Időzített lejátszás megállítás gomb. */
    private javax.swing.JButton stopButton;
    /** Algoritmus nevének címkéje. */
    private javax.swing.JLabel titleLabel;
    // End of variables declaration//GEN-END:variables
}
