package bogqaai;

import bogqaai.graph.*;
import bogqaai.graph.event.*;

/**
 * Az absztrakt gráfot kirajzoló felületek ősosztálya. Az osztály a
 * <i>javax.swing.JPanel</i> osztályból származik. Egy alapértelmezett működési
 * mechanizmust biztosít: az absztrakt gráf megváltozásakor és animálásakor
 * automatikusan újrarajzolja magát. A tényleges újrarajzolás módját a
 * leszármazott osztályok adják meg.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 * @param <T> Absztrakt gráf osztály
 */
public abstract class DrawerPanel<T extends AbstractGraph> extends javax.swing.JPanel {
    /** Háttér képe. */
    private java.awt.Image backImage = null;
    /** Absztrakt gráf képe. */
    private java.awt.Image bufferImage = null;

    /** Absztrakt gráfot tartalmazó objektum. */
    protected final T graph;
    /** Animáció aktuális értéke. */
    protected float percent = 1;

    /**
     * A rajzoló felület létrehozása, paramétere az absztrakt gráf.
     *
     * @param graph Absztrakt gráf
     */
    public DrawerPanel(T graph) {
        this.graph = graph;
        setPreferredSize(new java.awt.Dimension(0, 0));

        addComponentListener(new java.awt.event.ComponentAdapter() {
            @Override
            public void componentResized(java.awt.event.ComponentEvent evt) {
                backChanged();
            }
        });
        graph.addGraphListener(new GraphAdapter() {
            @Override
            public void changed(GraphEvent evt) {
                percent = 1;
                changed();
            }
            @Override
            public void animation(GraphEvent evt){
                percent = evt.getPercent();
                animation();
            }
        });
    }

    /**
     * A háttér és az absztrakt gráf képének kirajzolása, előtte azok szükség
     * szerinti frissítése. A metódus a befoglaló <i>JPanel</i> rajzolási
     * művelete.
     *
     * @param g A rajzolási cél
     */
    @Override
    public final void paint(java.awt.Graphics g) {
        if(bufferImage == null){
            if(backImage == null){
                backImage = createImage(getWidth(), getHeight());
                paintBack((java.awt.Graphics2D) backImage.getGraphics());
            }
            bufferImage = createImage(getWidth(), getHeight());
            paintGraphics((java.awt.Graphics2D) bufferImage.getGraphics());
        }
        g.drawImage(bufferImage, 0, 0, this);
    }

    /**
     * Az absztrakt gráf kirajzolása a leszármazott osztályokban.
     *
     * @param g A rajzolási cél
     */
    protected void paintGraphics(java.awt.Graphics2D g){
        g.drawImage(backImage, 0, 0, this);
    }

    /**
     * A háttér kirajzolása a leszármazott osztályokban.
     *
     * @param g A rajzolási cél
     */
    protected void paintBack(java.awt.Graphics2D g){
        g.setColor(java.awt.Color.WHITE);
        g.fillRect(0, 0, getWidth(), getHeight());
    }

    /**
     * Az absztrakt gráf megváltozásának eseménye.
     */
    public void changed(){
        bufferImage = null;
        repaint();
    }
    /**
     * A háttér megváltoztatása.
     */
    public void backChanged(){
        backImage = null;
        changed();
    }
    /**
     * Az absztrakt gráf animálásának eseménye.
     */
    public void animation(){}
}
