package bogqaai;

import bogqaai.graph.*;
import bogqaai.graph.event.*;

/**
 * A szerkesztőablakban megjelenő él szerkesztő felület. Grafikus,
 * <i>javax.swing.JPanel</i> leszármazott osztály, a NetBeans szerkesztőjével
 * készült.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class EdgePanel extends javax.swing.JPanel {
    /**
     * Az élt és két végpontját kirajzoló belső osztály. Az absztrakt gráfot
     * kirajzoló <i>DrawerPanel</i> osztályból származik. A rajzolási felület
     * közepén a szerkesztésre kijelölt élt és annak végpontjait jeleníti meg.
     */
    private class EdgeDrawerPanel extends DrawerPanel<Graph> {
        /**
         * A rajzoló osztály konstruktora, paramétere a gráf.
         *
         * @param graph Gráf
         */
        public EdgeDrawerPanel(Graph graph) {
            super(graph);
            setPreferredSize(new java.awt.Dimension(4*graph.getDrawerBorderX(), 4*graph.getDrawerBorderY()));
        }

        /**
         * Él és végpontjainak kirajzolása.
         *
         * @param g A rajzolási cél
         */
        @Override
        protected void paintGraphics(java.awt.Graphics2D g) {
            super.paintGraphics(g);
            Graph.Edge e = parent.getEdge();
            if(e != null){
                graph.drawEdge(g, e, getWidth()/4, getHeight()/2, 3*getWidth()/4, getHeight()/2, parent.isWeighted());
            }
        }
    }

    /** Hibás súly megadás esetén megjelenített üzenet. */
    private final String weightInvalidMessage;

    /** Koordináló objektum. */
    private final MainWindow parent;

    /**
     * Az él szerkesztő felület konstruktora, paramétere a koordináló objektum
     * és a gráf.
     *
     * @param parent Koordináló objektum
     * @param graph Gráf
     */
    public EdgePanel(MainWindow parent, Graph graph) {
        initComponents();
        // erőforrások
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Bundle");
        weightInvalidMessage = bundle.getString("EdgePanel.weight.invalid.message");

        // komponensek
        this.parent = parent;
        drawPane.setViewportView(new EdgeDrawerPanel(graph));
        graph.addGraphListener(new GraphAdapter() {
            @Override
            public void changed(GraphEvent evt) {
                Graph.Edge e = EdgePanel.this.parent.getEdge();
                if(e != null){
                    weightEdit.setText("" + e.getWeight());
                }
            }
        });
    }

    /**
     * A grafikus komponensek inicializálása a konstruktorban. Tartalmát a
     * NetBeans szerkesztője generálta.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.JToolBar titleBar = new javax.swing.JToolBar();
        javax.swing.JPanel titlePanel = new javax.swing.JPanel();
        javax.swing.JLabel titleLabel = new javax.swing.JLabel();
        javax.swing.JButton closeButton = new javax.swing.JButton();
        drawPane = new javax.swing.JScrollPane();
        javax.swing.JLabel weightLabel = new javax.swing.JLabel();
        weightEdit = new javax.swing.JTextField();
        javax.swing.JButton delButton = new javax.swing.JButton();

        setPreferredSize(new java.awt.Dimension(240, 300));
        addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                formFocusGained(evt);
            }
        });

        titleBar.setFloatable(false);
        titleBar.setRollover(true);

        titlePanel.setBackground(new java.awt.Color(255, 255, 255, 0));

        titleLabel.setFont(titleLabel.getFont().deriveFont((float)18));
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Bundle"); // NOI18N
        titleLabel.setText(bundle.getString("EdgePanel.titleLabel.text")); // NOI18N

        javax.swing.GroupLayout titlePanelLayout = new javax.swing.GroupLayout(titlePanel);
        titlePanel.setLayout(titlePanelLayout);
        titlePanelLayout.setHorizontalGroup(
            titlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(titlePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(titleLabel)
                .addContainerGap(59, Short.MAX_VALUE))
        );
        titlePanelLayout.setVerticalGroup(
            titlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(titlePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(titleLabel)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        titleBar.add(titlePanel);

        closeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/close.png"))); // NOI18N
        closeButton.setToolTipText(bundle.getString("EdgePanel.closeButton.toolTipText")); // NOI18N
        closeButton.setContentAreaFilled(false);
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });
        titleBar.add(closeButton);

        drawPane.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        drawPane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        drawPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

        weightLabel.setText(bundle.getString("EdgePanel.weightLabel.text")); // NOI18N

        weightEdit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                weightEditActionPerformed(evt);
            }
        });

        delButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/delete.png"))); // NOI18N
        delButton.setText(bundle.getString("EdgePanel.delButton.text")); // NOI18N
        delButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                delButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(drawPane)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(weightLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(weightEdit))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGap(0, 0, Short.MAX_VALUE)
                        .addComponent(delButton)))
                .addContainerGap())
            .addComponent(titleBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(titleBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(drawPane, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(weightLabel)
                    .addComponent(weightEdit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(delButton)
                .addContainerGap(91, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Élsúly megadásának eseménye.
     */
    private void weightEditActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_weightEditActionPerformed
        try{
            Graph.Edge e = parent.getEdge();
            if(e != null){
                e.setWeight(Integer.parseInt(weightEdit.getText()));
            }
        }catch(NumberFormatException e){
            parent.setStatus(weightInvalidMessage, weightEdit.getText());
        }
    }//GEN-LAST:event_weightEditActionPerformed

    /**
     * Él törlés gomb eseménye.
     */
    private void delButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_delButtonActionPerformed
        Graph.Edge e = parent.getEdge();
        if(e != null){
            e.remove();
            parent.selectNone();
        }
    }//GEN-LAST:event_delButtonActionPerformed

    /**
     * Bezárás gomb eseménye.
     */
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        parent.selectNone();
    }//GEN-LAST:event_closeButtonActionPerformed

    /**
     * Felület aktiválás eseménye: élsúly szerkesztő kijelölése.
     */
    private void formFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_formFocusGained
        weightEdit.requestFocusInWindow();
        weightEdit.selectAll();
    }//GEN-LAST:event_formFocusGained

    // Variables declaration - do not modify//GEN-BEGIN:variables
    /** Él és végpontjait kirajzoló felület befoglalója. */
    private javax.swing.JScrollPane drawPane;
    /** Élsúly szerkesztője. */
    private javax.swing.JTextField weightEdit;
    // End of variables declaration//GEN-END:variables
}
