package bogqaai;

import bogqaai.graph.*;

/**
 * A gráf megjelenítőben lévő, a gráfot kirajzoló osztály. Az absztrakt gráfot
 * kirajzoló <i>DrawerPanel</i> osztályból származik. A rajzolási felületen a
 * gráf csúcsait és éleit jeleníti meg, azok színezésével együtt. Az egérrel
 * végezhető szerkesztési műveletek ezen a felületen keresztül kerülnek
 * végrehajtásra.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class GraphDrawerPanel extends DrawerPanel<Graph> {
    /**
     * Egéresemények kezelésének belső osztálya.
     */
    private class MouseAdapter extends java.awt.event.MouseAdapter {
        /**
         * Egérgomb lenyomás eseménye.
         *
         * @param evt Esemény objektum
         */
        @Override
        public void mousePressed(java.awt.event.MouseEvent evt) {
            java.awt.Point p = evt.getPoint();
            p.translate(-graph.getDrawerBorderX(), -graph.getDrawerBorderY());
            mouseVertex = graph.findVertex(p.x, p.y);
            mousePos = null;
            if(!graph.isLocked()){
                if(evt.getButton() == java.awt.event.MouseEvent.BUTTON1){
                    if(evt.getClickCount()==2){
                        if(mouseVertex == null){
                            mouseVertex = graph.addVertex(p.x, p.y, parent.alignToGrid());
                        }else{
                            graph.removeVertex(mouseVertex);
                            mouseVertex = null;
                        }
                    }
                }else{
                    mousePos = p;
                }
                parent.selectVertex(mouseVertex);
            }else{
                if(evt.getButton() != java.awt.event.MouseEvent.BUTTON1){
                    mouseVertex = null;
                }
            }
            if(mouseVertex != null){
                mouseDelta = new java.awt.Point(mouseVertex.getX() - evt.getX(), mouseVertex.getY() - evt.getY());
            }
        }
        /**
         * Egérgomb felengedés eseménye.
         *
         * @param evt Esemény objektum
         */
        @Override
        public void mouseReleased(java.awt.event.MouseEvent evt) {
            if(mouseVertex != null){
                if(mousePos != null){
                    Graph.Vertex v = graph.findVertex(evt.getX() - graph.getDrawerBorderX(), evt.getY() - graph.getDrawerBorderY());
                    if(v != null && !v.equals(mouseVertex)){
                        parent.selectEdge(graph.addEdge(mouseVertex, v));
                    }else{
                        changed();
                    }
                }
            }
            mouseVertex = null;
            mousePos = null;
        }
        /**
         * Egér húzás eseménye.
         *
         * @param evt Esemény objektum
         */
        @Override
        public void mouseDragged(java.awt.event.MouseEvent evt) {
            if(mouseVertex != null){
                if(mousePos == null){
                    int x = mouseDelta.x + evt.getX();
                    int y = mouseDelta.y + evt.getY();
                    mouseVertex.setPos(x, y, parent.alignToGrid());
                }else{
                    mousePos = evt.getPoint();
                    mousePos.translate(-graph.getDrawerBorderX(), -graph.getDrawerBorderY());
                    changed();
                }
            }
        }
    }

    /** Koordináló objektum. */
    private final MainWindow parent;

    /** Egér esemény csúcsa. */
    private Graph.Vertex mouseVertex = null;
    /** Egér esemény helye csúcs mozgatásakor. */
    private java.awt.Point mouseDelta = null;
    /** Egér esemény helye. */
    private java.awt.Point mousePos = null;

    /**
     * Gráf rajzoló osztály konstruktora, paramétere a koordináló objektum
     * és a gráf.
     *
     * @param parent Koordináló objektum
     * @param graph Gráf
     */
    public GraphDrawerPanel(MainWindow parent, Graph graph) {
        super(graph);
        this.parent = parent;
        MouseAdapter adapter = new MouseAdapter();
        addMouseListener(adapter);
        addMouseMotionListener(adapter);
        setPreferredSize(new java.awt.Dimension(graph.getDrawerWidth(), graph.getDrawerHeight()));
    }

    /**
     * Gráf kirajzolása.
     *
     * @param g A rajzolási cél
     */
    @Override
    protected void paintGraphics(java.awt.Graphics2D g) {
        super.paintGraphics(g);
        Graph.Drawer drawer = graph.drawGraph(g, parent.isWeighted(), percent);
        if(mouseVertex != null && mousePos != null){
            drawer.drawEdge(mouseVertex.getX(), mouseVertex.getY(), mousePos.x, mousePos.y, null);
        }
    }

    /**
     * Háttér kirajzolása. A <i>Nézet</i> menü beállításainak megfelelően
     * rácspontokat jelenít meg a háttérben.
     *
     * @param g
     */
    @Override
    protected void paintBack(java.awt.Graphics2D g){
        super.paintBack(g);
        if(parent.showGrid()){
            graph.drawGrid(g, getWidth(), getHeight());
        }
    }

    /**
     * A gráf megváltozásának eseménye.
     */
    @Override
    public void changed(){
        setPreferredSize(new java.awt.Dimension(graph.getDrawerWidth(), graph.getDrawerHeight()));
        super.changed();
        revalidate();
    }

    /**
     * A gráf animálásának eseménye.
     */
    @Override
    public void animation(){
        changed();
    }
}
