package bogqaai;

import bogqaai.graph.*;
import bogqaai.graph.event.*;
import bogqaai.graph.algorithm.*;

/**
 * Az osztály feladata a programhoz tartozó főablak - önálló ablakként, vagy
 * applet formájában -, a felület egyes elemeinek és a gráfot tartalmazó
 * objektumnak a létrehozása, az ezek közötti kapcsolatok koordinálása. A
 * felület többi osztálya rajta keresztül képes üzenetet megjeleníteni az
 * állapotsorban, csúcsot és élt kijelölni, az aktuális kijelölést és a
 * menüpontok beállításait lekérdezni.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class MainWindow {
    /**
     * A böngészőbe ágyazható applet osztálya. Az osztály a
     * <i>javax.swing.JApplet</i> osztályból származik.
     */
    public static class Applet extends javax.swing.JApplet {
        /**
         * Az applet inicializálásakor lefutó kód. Létrehoz egy példányt a
         * koordináló osztályból, és megjeleníti a kezdőlapot.
         */
        @Override
        public void init(){
            MainWindow mainWindow = new MainWindow(this);
            setContentPane(mainWindow.startPanel);
        }
    }

    /**
     * A program belépési pontja.
     *
     * Létrehozza a főablakot (<i>javax.swing.JFrame</i>) és egy példányt a
     * koordináló osztályból. A főablak tartalma a parancssori paraméterektől
     * függ. Ha nincs paraméter, akkor a kezdőablakot jeleníti meg, különben
     * átugorja azt. Ha a paraméter „.”, akkor egy üres gráfot hoz létre,
     * egyébként betölti a megadott, gráfot tartalmazó fájlt.
     *
     * @param args Parancssori paraméterek
     */
    public static void main(String[] args){
        final String file = args.length > 0 ? args[0] : null;
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                javax.swing.JFrame frame = new javax.swing.JFrame();
                MainWindow mainWindow = new MainWindow(frame);
                frame.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
                frame.setTitle(mainWindow.title);
                frame.setJMenuBar(mainWindow.menuBar);
                frame.setContentPane(mainWindow.mainPanel);
                frame.pack();
                if(file == null){
                    frame.setJMenuBar(null);
                    frame.setContentPane(mainWindow.startPanel);
                }else if(!file.equals(".")){
                    mainWindow.internalOpenGraph(new java.io.File(file));
                }else{
                    mainWindow.newGraph();
                }
                frame.setLocationRelativeTo(null);
                frame.setMinimumSize(frame.getSize());
                frame.setVisible(true);
            }
        });
    }

    /** Főablak címsorának szövege. */
    private final String title;
    /** Névjegy szövege. */
    private final String aboutText;
    /** Fájl sikeres megnyitása esetén megjelenített üzenet. */
    private final String openSuccessMessage;
    /** Fájl sikertelen megnyitása esetén megjelenített üzenet. */
    private final String openFailMessage;
    /** Fájl sikeres mentése esetén megjelenített üzenet. */
    private final String saveSuccessMessage;
    /** Fájl sikertelen mentése esetén megjelenített üzenet. */
    private final String saveFailMessage;
    /** Algoritmus sikertelen betöltése esetén megjelenített üzenet. */
    private final String algorithmFailMessage;
    /** Mintagráf linkek formátuma a kezdőlapon. */
    private final String sampleFormat;

    /** Gráfot tartalmazó objektum. */
    private final Graph graph;
    /** Szerkesztésre kijelölt csúcs. */
    private Graph.Vertex vertex = null;
    /** Szerkesztésre kijelölt él. */
    private Graph.Edge edge = null;
    /** Szemléltetésre kiválasztott algoritmus. */
    private Algorithm algorithm = null;
    /** Kezdőlap állapota: értéke igaz, ha a kezdőlap látható. */
    private boolean start = true;

    /** Főablakot megvalósító <i>Component</i>. */
    private final java.awt.Component rootComponent;
    /** Főablakot megvalósító <i>RootPaneContainer</i>. */
    private final javax.swing.RootPaneContainer root;

    /** Kezdőlap. */
    private final StartPanel startPanel;
    /** Kezdőlap utáni váz. */
    private final MainPanel mainPanel;
    /** Gráfot kirajzoló felület. */
    private final GraphDrawerPanel graphPanel;
    /** Gráf szerkesztés összefoglaló. */
    private final EditorPanel editorPanel;
    /** Csúcs szerkesztő felület. */
    private final VertexPanel vertexPanel;
    /** Él szerkesztő felület. */
    private final EdgePanel edgePanel;
    /** Algoritmus vezérlő felület. */
    private final AlgorithmPanel algorithmPanel;
    /** Fájl megnyitás és mentés párbeszédablak gráfok tárolásához. */
    private final javax.swing.JFileChooser fileChooser;
    /** Fájl mentés párbeszédablak gráf és adatszerkezet képének mentéséhez. */
    private final javax.swing.JFileChooser pictureChooser;
    /** Súgóablak. */
    private final HelpDialog helpDialog;

    /** Menüsor. */
    private final javax.swing.JMenuBar menuBar;
    /** Mintagráfok menüje. */
    private final javax.swing.JMenu sampleMenu;
    /** Adatszerkezet mentése képként menüpont. */
    private final javax.swing.JMenuItem dataPictureMenuItem;
    /** Élsúlyok megjelenítése menüpont. */
    private final javax.swing.JCheckBoxMenuItem weightedMenuItem;
    /** Rácspontok megjelenítése menüpont. */
    private final javax.swing.JCheckBoxMenuItem gridMenuItem;
    /** Rácshoz igazítás menüpont. */
    private final javax.swing.JCheckBoxMenuItem alignMenuItem;
    /** Algoritmusok menü. */
    private final javax.swing.JMenu algMenu;
    /** Súgó menü. */
    private final javax.swing.JMenu helpMenu;

    /**
     * A koordináló osztály konstruktora.
     *
     * Létrehozza a gráfot tartalmazó objektumot és a felület egyes elemeit,
     * paramétere a főablakot megvalósító objektum. A felület elemeihez tartozó
     * szövegeket és az üzenetek szövegét a <i>bogqaai/Bundle.properties</i>
     * erőforrásfájlból tölti be, a párbeszédablakok feliratait az
     * <i>UIManager.properties</i>, a mintagráfok listáját a
     * <i>bogqaai/Samples.properties</i>, az algoritmusok listáját pedig a
     * <i>bogqaai/Algorithms.properties</i> fájlból.
     *
     * @param <T> Főablak típusa
     * @param root Főablak objektum
     */
    public <T extends java.awt.Component & javax.swing.RootPaneContainer> MainWindow(T root){
        javax.swing.UIManager.put("swing.boldMetal", Boolean.FALSE);
        // Erőforrások
        javax.swing.UIManager.getDefaults().addResourceBundle("UIManager");
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Bundle");
        title = bundle.getString("Title");
        aboutText = bundle.getString("About");
        openSuccessMessage = bundle.getString("MainWindow.open.success.message");
        openFailMessage = bundle.getString("MainWindow.open.fail.message");
        saveSuccessMessage = bundle.getString("MainWindow.save.success.message");
        saveFailMessage = bundle.getString("MainWindow.save.fail.message");
        algorithmFailMessage = bundle.getString("MainWindow.algorithm.fail.message");
        sampleFormat = bundle.getString("StartPanel.sample.format");

        // Menü
        menuBar = new javax.swing.JMenuBar();
        sampleMenu = new javax.swing.JMenu();
        dataPictureMenuItem = new javax.swing.JMenuItem();
        weightedMenuItem = new javax.swing.JCheckBoxMenuItem();
        gridMenuItem = new javax.swing.JCheckBoxMenuItem();
        alignMenuItem = new javax.swing.JCheckBoxMenuItem();
        algMenu = new javax.swing.JMenu();
        helpMenu = new javax.swing.JMenu();
        addMenu(bundle);
        addAlgorithms();
        javax.swing.JPanel samplePanel = addSamples();

        // Komponensek
        this.root = root;
        this.rootComponent = root;
        boolean isApplet = rootComponent instanceof java.applet.Applet;
        graph = new Graph();
        startPanel = new StartPanel(this, samplePanel);
        editorPanel = new EditorPanel();
        graphPanel = new GraphDrawerPanel(this, graph);
        vertexPanel = new VertexPanel(this, graph);
        edgePanel = new EdgePanel(this, graph);
        algorithmPanel = new AlgorithmPanel(this, graph);
        mainPanel = new MainPanel(graphPanel, editorPanel);
        helpDialog = new HelpDialog(rootComponent instanceof java.awt.Window ? (java.awt.Window)rootComponent : javax.swing.SwingUtilities.windowForComponent(rootComponent));
        fileChooser = new javax.swing.JFileChooser(isApplet ? null : ".");
        pictureChooser = new javax.swing.JFileChooser(isApplet ? null : ".");
        pictureChooser.setAcceptAllFileFilterUsed(false);
        pictureChooser.addChoosableFileFilter(new javax.swing.filechooser.FileNameExtensionFilter("PNG - Portable Network Graphics (*.png)", "png"));
        pictureChooser.addChoosableFileFilter(new javax.swing.filechooser.FileNameExtensionFilter("GIF - Graphics Interchange Format (*.gif)", "gif"));
        pictureChooser.addChoosableFileFilter(new javax.swing.filechooser.FileNameExtensionFilter("JPEG (*.jpg)", "jpg"));
        graph.addGraphListener(new GraphAdapter() {
            @Override
            public void changed(GraphEvent evt) {
                if(algorithm == null) setStatus("");
            }
        });
    }


    /**
     * Az állapotsor tartalmának beállítása.
     *
     * @param status Állapotsor új tartalma
     */
    public void setStatus(String status){
        mainPanel.setStatus(status);
    }
    /**
     * Az állapotsor tartalmának beállítása, egy megadott üzenet formátum és
     * annak paraméterei segítségével.
     *
     * @param status Üzenet formátum
     * @param arguments Üzenet paraméterek
     */
    public void setStatus(String status, Object ... arguments){
        setStatus(java.text.MessageFormat.format(status, arguments));
    }

    /**
     * Az <i>Élsúlyok megjelenítése</i> menüpont beállításának lekérdezése.
     *
     * @return Logikai érték: igaz, ha megjelennek az élsúlyok
     */
    public boolean isWeighted(){
        return algorithm == null ? weightedMenuItem.isSelected() : algorithm.isGraphWeighted();
    }
    /**
     * A <i>Rácspontok megjelenítése</i> menüpont beállításának lekérdezése.
     *
     * @return Logikai érték: igaz, ha láthatók a rácspontok
     */
    public boolean showGrid(){
        return alignMenuItem.isSelected() || gridMenuItem.isSelected();
    }
    /**
     * A <i>Rácshoz igazítás</i> menüpont beállításának lekérdezése.
     *
     * @return Logikai érték: igaz, ha a rácshoz igazítás engedélyezve van
     */
    public boolean alignToGrid(){
        return alignMenuItem.isSelected();
    }

    /**
     * A szerkesztésre kijelölt csúcs lekérdezése.
     *
     * @return A szerkesztésre kijelölt csúcs, vagy <i>null<i>, ha nincs
     */
    public Graph.Vertex getVertex(){
        return vertex;
    }
    /**
     * A szerkesztésre kijelölt él lekérdezése.
     *
     * @return A szerkesztésre kijelölt él, vagy <i>null<i>, ha nincs
     */
    public Graph.Edge getEdge(){
        return edge;
    }
    /**
     * A szemléltetésre kiválasztott algoritmus lekérdezése.
     *
     * @return A szemléltetésre kiválasztott algoritmus, vagy <i>null<i>
     */
    public Algorithm getAlgorithm(){
        return algorithm;
    }

    /**
     * Csúcs, él és algoritmus kijelölések megszüntetése, belső metódus.
     *
     * @param gray Ha értéke igaz, a gráfot szürkére színezi, különben feketére
     */
    private void internalSelectNone(boolean gray){
        if(start){
            start = false;
            root.getRootPane().setJMenuBar(menuBar);
            root.setContentPane(mainPanel);
            rootComponent.validate();
        }
        vertex = null;
        edge = null;
        if(algorithm != null){
            algorithmPanel.stop();
            algorithm = null;
        }
        graph.setLocked(false);
        graph.reset(gray);
        weightedMenuItem.setEnabled(true);
        dataPictureMenuItem.setEnabled(false);
    }
    /**
     * A szerkesztőablak tartalmának beállítása, belső metódus.
     *
     * @param panel A beállítandó <i>JPanel</i>
     */
    private void internalSetEditor(javax.swing.JPanel panel){
        graph.changed();
        mainPanel.setEditor(panel);
        panel.requestFocusInWindow();
    }

    /**
     * Csúcs, él és algoritmus kijelölések megszüntetése.
     */
    public void selectNone(){
        internalSelectNone(false);
        internalSetEditor(editorPanel);
    }
    /**
     * Csúcs kijelölése szerkesztésre.
     *
     * @param u A kijelölt csúcs, vagy <i>null</i>
     */
    public void selectVertex(Graph.Vertex u){
        if(u == null){
            selectNone();
        }else{
            internalSelectNone(true);
            vertex = u;
            vertex.set(0, 0);
            internalSetEditor(vertexPanel);
        }
    }
    /**
     * Él kijelölése szerkesztésre.
     *
     * @param e A kijelölt élt, vagy <i>null</i>
     */
    public void selectEdge(Graph.Edge e){
        if(e == null){
            selectNone();
        }else{
            internalSelectNone(true);
            edge = e;
            edge.getVertexU().set(0, 0);
            edge.getVertexV().set(0, 0);
            edge.set(0, 0);
            internalSetEditor(edgePanel);
        }
    }

    /**
     * A menü létrehozása.
     *
     * @param bundle Feliratokat tartalmazó erőforrás
     */
    private void addMenu(java.util.ResourceBundle bundle){
        javax.swing.JMenu menu;
        javax.swing.JMenuItem menuItem;
        // Fájl
        menu = new javax.swing.JMenu(bundle.getString("MainWindow.fileMenu.text"));
        menuBar.add(menu);
        // Új
        menuItem = new javax.swing.JMenuItem(bundle.getString("MainWindow.newMenuItem.text"));
        menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newGraph();
            }
        });
        menu.add(menuItem);
        // Megnyitás
        menuItem = new javax.swing.JMenuItem(bundle.getString("MainWindow.openMenuItem.text"));
        menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openGraph();
            }
        });
        menu.add(menuItem);
        // Mintagráf megnyitása
        sampleMenu.setText(bundle.getString("MainWindow.sampleMenu.text"));
        menu.add(sampleMenu);
        // Mentés
        menuItem = new javax.swing.JMenuItem(bundle.getString("MainWindow.saveMenuItem.text"));
        menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveGraph();
            }
        });
        menu.add(menuItem);
        // Elválasztó
        menu.add(new javax.swing.JPopupMenu.Separator());
        // Gráf mentése képként
        menuItem = new javax.swing.JMenuItem(bundle.getString("MainWindow.graphPictureMenuItem.text"));
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                savePicture(graphPanel);
            }
        });
        menu.add(menuItem);
        // Adatszerkezet mentése képként
        menuItem = new javax.swing.JMenuItem(bundle.getString("MainWindow.dataPictureMenuItem.text"));
        menuItem.setEnabled(false);
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                if(algorithm != null){
                    savePicture(algorithmPanel.getDataPanel());
                }
            }
        });
        menu.add(menuItem);

        // Nézet
        menu = new javax.swing.JMenu(bundle.getString("MainWindow.viewMenu.text"));
        menuBar.add(menu);
        // Élsúlyok megjelentése
        weightedMenuItem.setSelected(true);
        weightedMenuItem.setText(bundle.getString("MainWindow.weightedMenuItem.text"));
        weightedMenuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                graph.changed();
            }
        });
        menu.add(weightedMenuItem);
        // Elválasztó
        menu.add(new javax.swing.JPopupMenu.Separator());
        // Rácspontok megjelenítése
        gridMenuItem.setText(bundle.getString("MainWindow.gridMenuItem.text"));
        gridMenuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                graphPanel.backChanged();
            }
        });
        menu.add(gridMenuItem);
        // Rácshoz igazítás
        alignMenuItem.setText(bundle.getString("MainWindow.alignMenuItem.text"));
        alignMenuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                gridMenuItem.setEnabled(!alignMenuItem.isSelected());
                graphPanel.backChanged();
            }
        });
        menu.add(alignMenuItem);

        // Algoritmusok
        algMenu.setText(bundle.getString("MainWindow.algMenu.text"));
        menuBar.add(algMenu);
        // Alaphelyzet
        menuItem = new javax.swing.JMenuItem(bundle.getString("MainWindow.resetMenuItem.text"));
        menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.CTRL_MASK));
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectNone();
            }
        });
        algMenu.add(menuItem);
        // Elválasztó
        algMenu.add(new javax.swing.JPopupMenu.Separator());

        // Súgó
        helpMenu.setText(bundle.getString("MainWindow.helpMenu.text"));
        menuBar.add(helpMenu);
        // Névjegy
        menuItem = new javax.swing.JMenuItem(bundle.getString("MainWindow.aboutMenuItem.text"));
        menuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                javax.swing.JOptionPane.showMessageDialog(rootComponent, aboutText, title, javax.swing.JOptionPane.PLAIN_MESSAGE);
            }
        });
        helpMenu.add(menuItem);
        // Elválasztó
        helpMenu.add(new javax.swing.JPopupMenu.Separator());
    }

    /**
     * A mintagráfok listájának létrehozása.
     *
     * @return Mintagráfokat tartalmazó <i>JPanel</i>, a kezdőlapra
     */
    private javax.swing.JPanel addSamples(){
        javax.swing.JPanel panel = new javax.swing.JPanel();
        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(panel);
        panel.setLayout(layout);
        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);
        javax.swing.GroupLayout.ParallelGroup hGroup = layout.createParallelGroup();
        javax.swing.GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Samples");
        java.util.ArrayList<String> keylist = java.util.Collections.list(bundle.getKeys());
        java.util.Collections.sort(keylist);
        for(String key: keylist){
            final String value = bundle.getString(key);
            if(value.equals("-")){
                sampleMenu.add(new javax.swing.JPopupMenu.Separator());
                javax.swing.JSeparator separator = new javax.swing.JSeparator();
                hGroup.addComponent(separator);
                vGroup.addComponent(separator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE);
            }else{
                String[] pieces = value.split(":");
                if(pieces.length == 3){
                    final String name = pieces[0];
                    final String source = "/bogqaai/samples/" + pieces[1];
                    final boolean weighted = pieces[2].equals("1");
                    java.awt.event.ActionListener listener = new java.awt.event.ActionListener() {
                        @Override
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            openSample(name, source, weighted);
                        }
                    };
                    javax.swing.JMenuItem item = new javax.swing.JMenuItem(name);
                    item.addActionListener(listener);
                    sampleMenu.add(item);
                    javax.swing.JButton button = new javax.swing.JButton(java.text.MessageFormat.format(sampleFormat, name));
                    button.setBorder(null);
                    button.setBorderPainted(false);
                    button.setContentAreaFilled(false);
                    button.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
                    button.setHorizontalAlignment(javax.swing.JButton.LEFT);
                    button.addActionListener(listener);
                    hGroup.addComponent(button);
                    vGroup.addComponent(button);
                }
            }
        }

        layout.setHorizontalGroup(layout.createSequentialGroup().addGroup(hGroup));
        layout.setVerticalGroup(vGroup);
        return panel;
    }
    /**
     * Mintagráf megnyitása.
     *
     * @param name Mintagráf neve
     * @param source Mintagráf forráskódja
     * @param weighted Élsúlyok megjelenítése
     */
    private void openSample(String name, String source, boolean weighted){
        selectNone();
        try{
            java.io.InputStreamReader ir = null;
            try{
                ir = new java.io.InputStreamReader(this.getClass().getResourceAsStream(source));
                weightedMenuItem.setSelected(weighted);
                graph.open(ir);
                setStatus(openSuccessMessage, name);
            }finally{
                if(ir != null) ir.close();
            }
        }catch(Exception e){
            setStatus(openFailMessage, name, e.getMessage());
        }
    }

    /**
     * Az algoritmusok listájának létrehozása.
     */
    private void addAlgorithms(){
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Algorithms");
        java.util.ArrayList<String> keylist = java.util.Collections.list(bundle.getKeys());
        java.util.Collections.sort(keylist);
        for(String key: keylist){
            final String value = bundle.getString(key);
            if(value.equals("-")){
                algMenu.add(new javax.swing.JPopupMenu.Separator());
            }else{
                try{
                    final Class<? extends Algorithm> clazz = Class.forName("bogqaai.graph.algorithm." + value).asSubclass(Algorithm.class);
                    final Algorithm.Strings strings = Algorithm.getStrings(clazz);
                    javax.swing.JMenuItem algItem = new javax.swing.JMenuItem();
                    algItem.setText(strings.title);
                    algItem.addActionListener(new java.awt.event.ActionListener() {
                        @Override
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            openAlgorithm(clazz, strings);
                        }
                    });
                    algMenu.add(algItem);
                    javax.swing.JMenuItem helpItem = new javax.swing.JMenuItem();
                    helpItem.setText(strings.title);
                    helpItem.addActionListener(new java.awt.event.ActionListener() {
                        @Override
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            helpDialog.showHelp(strings.title, strings.help);
                        }
                    });
                    helpMenu.add(helpItem);
                }catch(Exception e){
                    javax.swing.JMenuItem item = new javax.swing.JMenuItem();
                    item.setText(value);
                    item.setEnabled(false);
                    algMenu.add(item);
                }
            }
        }
    }
    /**
     * Algoritmus betöltése.
     *
     * @param <T> Algoritmus típus
     * @param clazz Algoritmus osztály változó
     * @param strings Algoritmus adatait leíró rekord
     */
    private <T extends Algorithm> void openAlgorithm(Class<T> clazz, Algorithm.Strings strings){
        internalSelectNone(false);
        try{
            algorithm = clazz.getConstructor(Graph.class).newInstance(graph);
            weightedMenuItem.setEnabled(false);
            dataPictureMenuItem.setEnabled(true);
            algorithmPanel.start(strings);
            internalSetEditor(algorithmPanel);
        }catch(Exception e){
            graph.setLocked(false);
            internalSetEditor(editorPanel);
            setStatus(algorithmFailMessage, e.getMessage());
        }
    }

    /**
     * Üres gráf létrehozása.
     */
    public void newGraph(){
        selectNone();
        graph.clear();
    }
    /**
     * Fájl megnyitás párbeszédablak megjelenítése és gráf betöltése fájlból.
     */
    public void openGraph(){
        if(fileChooser.showOpenDialog(rootComponent) == javax.swing.JFileChooser.APPROVE_OPTION){
            internalOpenGraph(fileChooser.getSelectedFile());
        }
    }
    /**
     * Gráf betöltése fájlból, belső metódus.
     *
     * @param file Beolvasandó fájl
     */
    private void internalOpenGraph(java.io.File file){
        selectNone();
        try{
            java.io.FileReader fr = null;
            try{
                fr = new java.io.FileReader(file);
                graph.open(fr);
                setStatus(openSuccessMessage, file.getName());
            }finally{
                if(fr != null) fr.close();
            }
        }catch(Exception e){
            setStatus(openFailMessage, file.getName(), e.getMessage());
        }
    }
    /**
     * Fájl mentés párbeszédablak megjelenítése és gráf mentése fájlba.
     */
    private void saveGraph(){
        if(fileChooser.showSaveDialog(rootComponent) == javax.swing.JFileChooser.APPROVE_OPTION){
            setStatus("");
            java.io.File file = fileChooser.getSelectedFile();
            try{
                java.io.FileWriter fw = null;
                try{
                    fw = new java.io.FileWriter(file);
                    graph.save(fw);
                    setStatus(saveSuccessMessage, file.getName());
                }finally{
                    if(fw != null) fw.close();
                }
            }catch(Exception e){
                setStatus(saveFailMessage, file.getName(), e.getMessage());
            }
        }
    }
    /**
     * Fájl mentés párbeszédablak megjelenítése és kép - gráf vagy adatszerkezet
     * - mentése fájlba.
     *
     * @param panel A gráfot vagy adatszerkezetet kirajzoló felület
     */
    private void savePicture(DrawerPanel panel){
        if(panel != null && pictureChooser.showSaveDialog(rootComponent) == javax.swing.JFileChooser.APPROVE_OPTION){
            setStatus("");
            java.io.File file = pictureChooser.getSelectedFile();
            try{
                javax.swing.filechooser.FileNameExtensionFilter filter = (javax.swing.filechooser.FileNameExtensionFilter)pictureChooser.getFileFilter();
                String extension = filter.getExtensions()[0];
                String path = file.getAbsolutePath();
                if(!path.endsWith("." + extension)){
                    file = new java.io.File(path + "." + extension);
                }
                java.awt.image.BufferedImage image = new java.awt.image.BufferedImage(panel.getWidth(), panel.getHeight(), java.awt.image.BufferedImage.TYPE_INT_RGB);
                panel.paint((java.awt.Graphics2D) image.getGraphics());
                javax.imageio.ImageIO.write(image, extension, file);
                setStatus(saveSuccessMessage, file.getName());
            }catch(Exception e){
                setStatus(saveFailMessage, file.getName(), e.getMessage());
            }
        }
    }
}
