package bogqaai;

import bogqaai.graph.*;
import bogqaai.graph.event.*;

/**
 * A szerkesztőablakban megjelenő csúcs szerkesztő felület. Grafikus,
 * <i>javax.swing.JPanel</i> leszármazott osztály, a NetBeans szerkesztőjével
 * készült.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class VertexPanel extends javax.swing.JPanel {
    /**
     * A csúcsot kirajzoló belső osztály. Az absztrakt gráfot
     * kirajzoló <i>DrawerPanel</i> osztályból származik. A rajzolási felület
     * közepén a szerkesztésre kijelölt csúcsot jeleníti meg.
     */
    private class VertexDrawerPanel extends DrawerPanel<Graph> {
        /**
         * A rajzoló osztály konstruktora, paramétere a gráf.
         *
         * @param graph Gráf
         */
        public VertexDrawerPanel(Graph graph) {
            super(graph);
            setPreferredSize(new java.awt.Dimension(2*graph.getDrawerBorderX(), 2*graph.getDrawerBorderY()));
        }

        /**
         * Csúcs kirajzolása.
         *
         * @param g A rajzolási cél
         */
        @Override
        protected void paintGraphics(java.awt.Graphics2D g) {
            super.paintGraphics(g);
            Graph.Vertex u = parent.getVertex();
            if(u != null){
                graph.drawVertex(g, u, getWidth()/2, getHeight()/2);
            }
        }
    }

    /** Hibás sorszám megadás esetén megjelenített üzenet. */
    private final String idInvalidMessage;
    /** Ütköző sorszám megadás esetén megjelenített üzenet. */
    private final String idFailMessage;
    /** Hibás koordináta megadás esetén megjelenített üzenet. */
    private final String posInvalidMessage;

    /** Koordináló objektum. */
    private final MainWindow parent;

    /**
     * A csúcs szerkesztő felület konstruktora, paramétere a koordináló objektum
     * és a gráf.
     *
     * @param parent Koordináló objektum
     * @param graph Gráf
     */
    public VertexPanel(MainWindow parent, Graph graph) {
        initComponents();
        // erőforrások
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Bundle");
        idInvalidMessage = bundle.getString("VertexPanel.id.invalid.message");
        idFailMessage = bundle.getString("VertexPanel.id.fail.message");
        posInvalidMessage = bundle.getString("VertexPanel.pos.invalid.message");

        // komponensek
        this.parent = parent;
        drawPane.setViewportView(new VertexDrawerPanel(graph));
        graph.addGraphListener(new GraphAdapter() {
            @Override
            public void changed(GraphEvent evt) {
                Graph.Vertex u = VertexPanel.this.parent.getVertex();
                if(u != null){
                    idEdit.setText("" + u.getId());
                    xEdit.setText("" + u.getX());
                    yEdit.setText("" + u.getY());
                }
            }
        });
    }

    /**
     * A grafikus komponensek inicializálása a konstruktorban. Tartalmát a
     * NetBeans szerkesztője generálta.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.JToolBar titleBar = new javax.swing.JToolBar();
        javax.swing.JPanel titlePanel = new javax.swing.JPanel();
        javax.swing.JLabel titleLabel = new javax.swing.JLabel();
        javax.swing.JButton closeButton = new javax.swing.JButton();
        drawPane = new javax.swing.JScrollPane();
        javax.swing.JLabel idLabel = new javax.swing.JLabel();
        idEdit = new javax.swing.JTextField();
        javax.swing.JLabel xLabel = new javax.swing.JLabel();
        xEdit = new javax.swing.JTextField();
        javax.swing.JLabel yLabel = new javax.swing.JLabel();
        yEdit = new javax.swing.JTextField();
        javax.swing.JButton delButton = new javax.swing.JButton();

        setPreferredSize(new java.awt.Dimension(240, 300));
        addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                formFocusGained(evt);
            }
        });

        titleBar.setFloatable(false);
        titleBar.setRollover(true);

        titlePanel.setBackground(new java.awt.Color(255, 255, 255, 0));

        titleLabel.setFont(titleLabel.getFont().deriveFont((float)18));
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/Bundle"); // NOI18N
        titleLabel.setText(bundle.getString("VertexPanel.titleLabel.text")); // NOI18N

        javax.swing.GroupLayout titlePanelLayout = new javax.swing.GroupLayout(titlePanel);
        titlePanel.setLayout(titlePanelLayout);
        titlePanelLayout.setHorizontalGroup(
            titlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(titlePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(titleLabel)
                .addContainerGap(30, Short.MAX_VALUE))
        );
        titlePanelLayout.setVerticalGroup(
            titlePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(titlePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(titleLabel)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        titleBar.add(titlePanel);

        closeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/close.png"))); // NOI18N
        closeButton.setToolTipText(bundle.getString("VertexPanel.closeButton.toolTipText")); // NOI18N
        closeButton.setContentAreaFilled(false);
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });
        titleBar.add(closeButton);

        drawPane.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        drawPane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        drawPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

        idLabel.setText(bundle.getString("VertexPanel.idLabel.text")); // NOI18N

        idEdit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                idEditActionPerformed(evt);
            }
        });

        xLabel.setText(bundle.getString("VertexPanel.xLabel.text")); // NOI18N

        xEdit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xEditActionPerformed(evt);
            }
        });

        yLabel.setText(bundle.getString("VertexPanel.yLabel.text")); // NOI18N

        yEdit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                yEditActionPerformed(evt);
            }
        });

        delButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/bogqaai/icons/delete.png"))); // NOI18N
        delButton.setText(bundle.getString("VertexPanel.delButton.text")); // NOI18N
        delButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                delButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(titleBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(drawPane)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGap(0, 0, Short.MAX_VALUE)
                        .addComponent(delButton))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(idLabel)
                            .addComponent(yLabel)
                            .addComponent(xLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(yEdit)
                            .addComponent(xEdit)
                            .addComponent(idEdit))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(titleBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(drawPane, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(idLabel)
                    .addComponent(idEdit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(xLabel)
                    .addComponent(xEdit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(yLabel)
                    .addComponent(yEdit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(delButton)
                .addContainerGap(39, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Sorszám megadásának eseménye.
     */
    private void idEditActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_idEditActionPerformed
        try{
            Graph.Vertex u = parent.getVertex();
            if(u != null){
                if(!u.setId(Integer.parseInt(idEdit.getText()))){
                    parent.setStatus(idFailMessage);
                }
            }
        }catch(NumberFormatException e){
            parent.setStatus(idInvalidMessage, idEdit.getText());
        }
    }//GEN-LAST:event_idEditActionPerformed

    /**
     * Csúcs törlés gomb eseménye.
     */
    private void delButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_delButtonActionPerformed
        Graph.Vertex u = parent.getVertex();
        if(u != null){
            u.remove();
            parent.selectNone();
        }
    }//GEN-LAST:event_delButtonActionPerformed

    /**
     * Felület aktiválás eseménye: sorszám szerkesztő kijelölése.
     */
    private void formFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_formFocusGained
        idEdit.requestFocusInWindow();
        idEdit.selectAll();
    }//GEN-LAST:event_formFocusGained

    /**
     * Bezárás gomb eseménye.
     */
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        parent.selectNone();
    }//GEN-LAST:event_closeButtonActionPerformed

    /**
     * X koordináta megadás eseménye.
     */
    private void xEditActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_xEditActionPerformed
        try{
            Graph.Vertex u = parent.getVertex();
            if(u != null){
                u.setPos(Integer.parseInt(xEdit.getText()), u.getY(), false);
            }
        }catch(NumberFormatException e){
            parent.setStatus(posInvalidMessage, xEdit.getText());
        }
    }//GEN-LAST:event_xEditActionPerformed

    /**
     * Y koordináta megadás eseménye.
     */
    private void yEditActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_yEditActionPerformed
        try{
            Graph.Vertex u = parent.getVertex();
            if(u != null){
                u.setPos(u.getX(), Integer.parseInt(yEdit.getText()), false);
            }
        }catch(NumberFormatException e){
            parent.setStatus(posInvalidMessage, yEdit.getText());
        }
    }//GEN-LAST:event_yEditActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    /** Csúcsot kirajzoló felület befoglalója. */
    private javax.swing.JScrollPane drawPane;
    /** Sorszám szerkesztője. */
    private javax.swing.JTextField idEdit;
    /** X koordináta szerkesztője. */
    private javax.swing.JTextField xEdit;
    /** Y koordináta szerkesztője. */
    private javax.swing.JTextField yEdit;
    // End of variables declaration//GEN-END:variables
}