package bogqaai.graph;

import bogqaai.graph.event.*;

/**
 * Az absztrakt gráf ősosztály, a grafikusan kirajzolható gráf és
 * adatszerkezetek őse. Tartalmazza az absztrakt gráf elem ősosztályt, egy
 * csúcsot és élt rajzoló osztályt és a rajzoláshoz szükséges konstansokat. Az
 * osztály biztosítja a gráf megváltozás értesítések eseményvezérelt
 * mechanizmusát: a grafikus komponensek értesítést kérhetnek a gráf
 * megváltozásáról illetve kiválthatják az eseményt a gráf elemeinek módosítása
 * után. Az esemény lehet egyszerű vagy animált.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public abstract class AbstractGraph {
    /**
     * A csúcsok és élek rajzolásához használható színek.
     */
    public interface Color{
        /** Fekete. */
        public static final int BLACK = 0;
        /** Szürke. */
        public static final int GRAY = 1;
        /** Fehér. */
        public static final int WHITE = 2;
        /** Piros. */
        public static final int RED = 3;
        /** Zöld. */
        public static final int GREEN = 4;
        /** Kék. */
        public static final int BLUE = 5;
    }
    /**
     * A csúcsok rajzolásához használható kitöltés típusok: nincs kitöltés,
     * a csúcs színével megegyező a kitöltés vagy a csúcs sárgával megjelölt.
     */
    public interface Fill{
        /** Nincs kitöltés. */
        public static final int EMPTY = 0;
        /** Kitöltött a csúcs színével. */
        public static final int FILLED = 1;
        /** Sárgával megjelölt. */
        public static final int MARKED = 2;
    }
    /**
     * A csúcsok és élek rajzolásához használható animáció típusok: nincs
     * animáció, színátmenetes, késleltetett, élek esetén adott irányban
     * színátmenetes animáció.
     */
    public interface Animation{
        public static final int NONE = 0;
        public static final int NORMAL = 1;
        public static final int DELAYED = 2;
        public static final int LTR = 3;
        public static final int RTL = 4;
    }

    /**
     * A csúcsokat és éleket kirajzoló absztrakt osztály. A rajzolás alapvető
     * mechanizmusát biztosítja: csúcsot, élt és rácspontokat képes rajzolni a
     * megfelelő paraméterekkel, és tartalmazza a rajzoláshoz szükséges
     * adatokat.
     */
    public static abstract class Drawer {
        /** Szegély vastagság. */
        public static final int BORDER = 4;

        /** Sárgával megjelölt csúcsok kitöltési színe. */
        private static final java.awt.Color YELLOW = new java.awt.Color(255, 255, 128);
        /** Üres csúcsok kitöltési színe. */
        private static final java.awt.Color WHITE = java.awt.Color.WHITE;
        /** Lehetséges színek. */
        private static final java.awt.Color[] COLORS = {
            java.awt.Color.BLACK, java.awt.Color.LIGHT_GRAY, java.awt.Color.WHITE,
            java.awt.Color.RED, java.awt.Color.GREEN, java.awt.Color.BLUE
        };
        /** Lehetséges kitöltő színek. */
        private static final java.awt.Color[] FILLS = {
            java.awt.Color.LIGHT_GRAY, new java.awt.Color(224, 224, 224), java.awt.Color.BLACK,
            new java.awt.Color(255, 224, 224), new java.awt.Color(224, 255, 224), new java.awt.Color(224, 224, 255)
        };

        /** Tényleges vízszintes szegély. */
        private final int borderX;
        /** Tényleges függőleges szegély. */
        private final int borderY;
        /** Csúcsokat megjelenítő ellipszis vízszintes nagysága. */
        private final int radiusX;
        /** Csúcsokat megjelenítő ellipszis függőleges nagysága. */
        private final int radiusY;
        /** A csúcsok szövege több sort tartalmaz-e. */
        private final boolean multiLineVertex;
        /** A csúcsok körvonalának vastagsága. */
        private final java.awt.Stroke vertexStroke;
        /** Az élek vastagsága. */
        private final java.awt.Stroke edgeStroke;
        /** A csúcsok betűtípusa. */
        private final java.awt.Font vertexFont;
        /** Az élek betűtípusa. */
        private final java.awt.Font edgeFont;
        /** A csúcsok betűtípusának leírója. */
        private final java.awt.FontMetrics vertexFm;
        /** Az élek betűtípusának leírója. */
        private final java.awt.FontMetrics edgeFm;
        /** A csúcsok betűinek felső margója. */
        private final int vertexFmTop;
        /** Az élek betűinek magassága. */
        private final int edgeFmHeight;
        /** Az élek betűinek emelkedése. */
        private final int edgeFmAscent;

        /** A rajzolási cél. */
        private java.awt.Graphics2D g;

        /**
         * A rajzoló osztály konstruktora, paraméterei a rajzolási cél és a
         * rajzolás beállításai.
         *
         * @param g A rajzolási cél
         * @param border Szegély használata
         * @param radiusX Csúcsokat megjelenítő ellipszis vízszintes nagysága
         * @param radiusY Csúcsokat megjelenítő ellipszis függőleges nagysága
         * @param multiLineVertex A csúcsok szöveges több sort tartalmaz-e
         * @param vertexFontSize A csúcsok betűinek mérete
         * @param edgeFontSize Az élek betűinek mérete
         * @param vertexWidth A csúcsok körvonalának vastagsága
         * @param edgeWidth Az élek vastagsága
         */
        public Drawer(java.awt.Graphics2D g, boolean border, int radiusX, int radiusY, boolean multiLineVertex, int vertexFontSize, int edgeFontSize, int vertexWidth, int edgeWidth){
            this.g = g;
            this.borderX = border ? BORDER + radiusX : 0;
            this.borderY = border ? BORDER + radiusY : 0;
            this.radiusX = radiusX;
            this.radiusY = radiusY;
            this.multiLineVertex = multiLineVertex;
            vertexStroke = new java.awt.BasicStroke(vertexWidth);
            edgeStroke = new java.awt.BasicStroke(edgeWidth);
            vertexFont = g.getFont().deriveFont(java.awt.Font.PLAIN, vertexFontSize);
            edgeFont = g.getFont().deriveFont(java.awt.Font.PLAIN, edgeFontSize);
            vertexFm = g.getFontMetrics(vertexFont);
            edgeFm = g.getFontMetrics(edgeFont);
            vertexFmTop = vertexFm.getHeight()/2 - vertexFm.getAscent();
            edgeFmHeight = edgeFm.getHeight();
            edgeFmAscent = edgeFm.getAscent();
            g.setRenderingHint(java.awt.RenderingHints.KEY_ANTIALIASING, java.awt.RenderingHints.VALUE_ANTIALIAS_ON);
            g.setRenderingHint(java.awt.RenderingHints.KEY_TEXT_ANTIALIASING, java.awt.RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        }

        /**
         * Kitöltési szín meghatározása csúcshoz.
         *
         * @param color Szín
         * @param fill Kitöltés
         * @return Kitöltési szín
         */
        protected final java.awt.Color fillColor(int color, int fill){
            switch(fill){
                case AbstractGraph.Fill.FILLED:
                    return FILLS[color];
                case AbstractGraph.Fill.MARKED:
                    return YELLOW;
                default:
                    return WHITE;
            }
        }
        /**
         * Szín keverés színátmenetes rajzoláshoz.
         *
         * @param oldColor Régi szín
         * @param color Aktuális szín
         * @param percent Színátmenet mértéke
         * @return Kevert szín
         */
        protected final java.awt.Color mixColor(java.awt.Color oldColor, java.awt.Color color, float percent){
            int cr = (int)(oldColor.getRed()*(1-percent) + color.getRed()*percent);
            int cg = (int)(oldColor.getGreen()*(1-percent) + color.getGreen()*percent);
            int cb = (int)(oldColor.getBlue()*(1-percent) + color.getBlue()*percent);
            return new java.awt.Color(cr, cg, cb);
        }

        /**
         * Csúcs rajzolása megadott színnel, kitöltéssel és felirattal.
         *
         * @param x X koordináta
         * @param y Y koordináta
         * @param color Szín
         * @param fill Kitöltés
         * @param text Felirat
         */
        public final void drawVertex(int x, int y, java.awt.Color color, java.awt.Color fill, String text){
            x += borderX;
            y += borderY;
            g.setStroke(vertexStroke);
            g.setFont(vertexFont);
            g.setColor(fill == null ? WHITE : fill);
            g.fillOval(x - radiusX, y - radiusY, 2*radiusX, 2*radiusY);
            g.setColor(color == null ? COLORS[0] : color);
            g.drawOval(x - radiusX, y - radiusY, 2*radiusX, 2*radiusY);
            if(multiLineVertex){
                String[] lines = text.split("\n", 2);
                String line1 = lines[0];
                String line2 = lines.length > 0 ? lines[1] : "";
                g.drawString(line1, x - vertexFm.stringWidth(line1)/2, y - vertexFmTop - radiusY/2);
                g.drawString(line2, x - vertexFm.stringWidth(line2)/2, y - vertexFmTop + radiusY/2);
            }else{
                g.drawString(text, x - vertexFm.stringWidth(text)/2, y - vertexFmTop);
                // Horz center: x - r + (2*r - width)/2 = x - width/2
                // Vert center: y - r + (2*y - height)/2 + ascent = y - height/2 + ascent
            }
        }

        /**
         * Él rajzolása megadott színnel.
         *
         * @param x1 Kezdőpont X koordináta
         * @param y1 Kezdőpont Y koordináta
         * @param x2 Végpont X koordináta
         * @param y2 Végpont Y koordináta
         * @param color Szín
         */
        public final void drawEdge(int x1, int y1, int x2, int y2, java.awt.Color color){
            g.setStroke(edgeStroke);
            g.setColor(color == null ? COLORS[0] : color);
            g.drawLine(x1 + borderX, y1 + borderY, x2 + borderX, y2 + borderY);
        }
        /**
         * Él feliratának rajzolása megadott színnel.
         *
         * @param x1 Kezdőpont X koordináta
         * @param y1 Kezdőpont Y koordináta
         * @param x2 Végpont X koordináta
         * @param y2 Végpont Y koordináta
         * @param color Szín
         * @param text Felirat
         */
        public final void drawEdgeLabel(int x1, int y1, int x2, int y2, java.awt.Color color, String text){
            int textWidth = edgeFm.stringWidth(text);
            int x = (x1+x2)/2 - textWidth/2 - 1 + borderX;
            int y = (y1+y2)/2 - edgeFmHeight/2 + borderY;
            g.setFont(edgeFont);
            g.setColor(WHITE);
            g.fillRect(x, y, textWidth + 2, edgeFmHeight);
            g.setColor(color == null ? COLORS[0] : color);
            g.drawString(text, x + 1, y + edgeFmAscent);
        }

        /**
         * Rácspontok rajzolása, megadott szélességben és magasságban.
         *
         * @param width Felület szélessége
         * @param height Felület magassága
         */
        public final void drawGrid(int width, int height){
            g.setColor(COLORS[0]);
            for(int x=borderX; x<width-borderX; x+=radiusX){
                for(int y=borderY; y<height-borderY; y+=radiusY){
                    g.drawRect(x, y, 0, 0);
                }
            }
        }
    }

    /**
     * Az absztrakt gráf elem ősosztály, a tényleges csúcs és él osztályok őse.
     * Tartalmazza a gráf elemek közös adattagjait, a megváltoztatásukhoz,
     * kirajzolásukhoz és animálásukhoz szükséges metódusokat. Minden gráf elem
     * rendelkezik színnel és kitöltéssel, tárolja az aktuális animáció típusát
     * és egy <i>data</i> adattagot további információk hozzárendelésére. Ezen
     * adatok lekérdezése és módosítása mellett az elemnek lekérdezhető a
     * felirata, és kirajzolható csúcsként vagy élként.
     */
    public abstract class GraphElement {
        /** Szín, alapértelmezetten fekete. */
        private int color = 0;
        /** Kitöltés, alapértelmezetten üres. */
        private int fill = 0;
        /** Régi szín, animáció esetén. */
        private int oldColor = 0;
        /** Régi kitöltés, animáció esetén. */
        private int oldFill = 0;
        /** Animáció típusa. */
        private int animationType = 0;

        /** Speciálisan hozzárendelt adatok. */
        public Object data = null;

        /**
         * Szín lekérdezése.
         *
         * @return Szín
         */
        public final int getColor(){
            return color;
        }
        /**
         * Kitöltés típusának lekérdezése.
         *
         * @return Kitöltés
         */
        public final int getFill(){
            return fill;
        }
        /**
         * Szín és kitöltés megváltoztatása.
         *
         * @param color Új szín
         * @param fill Új kitöltés
         */
        public final void set(int color, int fill){
            set(color, fill, 0);
        }
        /**
         * Szín és kitöltés megváltoztatása animáltan.
         *
         * @param color Új szín
         * @param fill Új kitöltés
         * @param animationType Animáció típusa
         */
        public final void set(int color, int fill, int animationType){
            if(animationType == 0){
                oldColor = this.color = color;
                oldFill = this.fill = fill;
                this.animationType = 0;
            }else{
                oldColor = this.color;
                this.color = color;
                oldFill = this.fill;
                this.fill = fill;
                this.animationType = animationType;
                changes.add(this);
            }
        }

        /**
         * Az elem adatainak megváltozásakor, a kirajzolás illetve animáció
         * utáni jelzés.
         */
        public final void changed(){
            animationType = 0;
            oldColor = color;
            oldFill = fill;
        }

        /**
         * Az elem kirajzolása csúcsként, egy rajzoló objektum segítségével.
         *
         * @param drawer Rajzoló objektum
         * @param x X koordináta
         * @param y Y koordináta
         */
        public void drawVertex(Drawer drawer, int x, int y){
            drawer.drawVertex(x, y, null, null, toString());
        }
        /**
         * Az elem animált kirajzolása csúcsként, egy rajzoló objektum
         * segítségével.
         *
         * @param drawer Rajzoló objektum
         * @param x X koordináta
         * @param y Y koordináta
         * @param percent Animáció mértéke
         */
        public void drawVertex(Drawer drawer, int x, int y, float percent){
            java.awt.Color dcolor;
            java.awt.Color dfill;
            int type = percent < 1 ? animationType : 0;
            switch(type){
                case Graph.Animation.NORMAL:
                    dcolor = drawer.mixColor(Drawer.COLORS[oldColor], Drawer.COLORS[color], percent);
                    dfill = drawer.mixColor(drawer.fillColor(oldColor, oldFill), drawer.fillColor(color, fill), percent);
                    break;
                case Graph.Animation.DELAYED:
                    dcolor = Drawer.COLORS[oldColor];
                    dfill = drawer.fillColor(oldColor, oldFill);
                    break;
                default:
                    dcolor = Drawer.COLORS[color];
                    dfill = drawer.fillColor(color, fill);
                    break;
            }
            drawer.drawVertex(x, y, dcolor, dfill, toString());
        }

        /**
         * Az elem kirajzolása élként, egy rajzoló objektum segítségével.
         *
         * @param drawer Rajzoló objektum
         * @param x1 Kezdőpont X koordináta
         * @param y1 Kezdőpont Y koordináta
         * @param x2 Végpont X koordináta
         * @param y2 Végpont Y koordináta
         * @param weighted Súly megjelenítése feliratként
         */
        public void drawEdge(Drawer drawer, int x1, int y1, int x2, int y2, boolean weighted){
            drawer.drawEdge(x1, y1, x2, y2, null);
            if(weighted){
                drawer.drawEdgeLabel(x1, y1, x2, y2, null, toString());
            }
        }
        /**
         * Az elem animált kirajzolása élként, egy rajzoló objektum segítségével.
         *
         * @param drawer Rajzoló objektum
         * @param x1 Kezdőpont X koordináta
         * @param y1 Kezdőpont Y koordináta
         * @param x2 Végpont X koordináta
         * @param y2 Végpont Y koordináta
         * @param weighted Súly megjelenítése feliratként
         * @param percent Animáció mértéke
         */
        public void drawEdge(Drawer drawer, int x1, int y1, int x2, int y2, boolean weighted, float percent){
            java.awt.Color dcolor;
            int type = percent < 1 ? animationType : 0;
            switch(type){
                case Graph.Animation.RTL:
                    int t = x1;
                    x1 = x2;
                    x2 = t;
                    t = y1;
                    y1 = y2;
                    y2 = t;
                case Graph.Animation.LTR:
                    dcolor = Drawer.COLORS[percent > 0.5 ? color : oldColor];
                    //// x1 + (x2-x1)*percent = x2*t + x1*(1-percent)
                    drawer.drawEdge(x1, y1, (int)(x2*percent + x1*(1-percent)), (int)(y2*percent + y1*(1-percent)), Drawer.COLORS[color]);
                    drawer.drawEdge((int)(x2*percent + x1*(1-percent)), (int)(y2*percent + y1*(1-percent)), x2, y2, Drawer.COLORS[oldColor]);
                    break;
                default:
                    switch(type){
                        case Graph.Animation.NORMAL:
                            dcolor = drawer.mixColor(Drawer.COLORS[oldColor], Drawer.COLORS[color], percent);
                            break;
                        case Graph.Animation.DELAYED:
                            dcolor = Drawer.COLORS[oldColor];
                            break;
                        default:
                            dcolor = Drawer.COLORS[color];
                    }
                    drawer.drawEdge(x1, y1, x2, y2, dcolor);
            }
            if(weighted){
                drawer.drawEdgeLabel(x1, y1, x2, y2, dcolor, toString());
            }
        }

        /**
         * Felirat szövege a leszármazott osztályokban.
         *
         * @return Felirat
         */
        @Override
        public String toString(){
            return "";
        }
    }

    /** Gráf megváltozás eseményre várakozók listája. */
    private final java.util.LinkedList<GraphListener> listeners = new java.util.LinkedList<GraphListener>();
    /** Megváltoztatott gráf elemek listája. */
    private final java.util.LinkedList<GraphElement> changes = new java.util.LinkedList<GraphElement>();

    /** Gráf szélessége rajzoláskor. */
    protected int drawerWidth = 0;
    /** Gráf magassága rajzoláskor. */
    protected int drawerHeight = 0;
    /** Vízszintes szegély vastagsága rajzoláskor. */
    protected int drawerBorderX = Drawer.BORDER;
    /** Vízszintes szegély vastagsága rajzoláskor. */
    protected int drawerBorderY = Drawer.BORDER;

    /**
     * A gráf beállításainak törlése a leszármazott osztályokban.
     */
    protected void internalClear(){}
    /**
     * A gráf beállításainak törlése, gráf esemény kiváltása.
     */
    public final void clear(){
        internalClear();
        changes.clear();
        changed();
    }

    /**
     * Feliratkozás a gráf megváltozásának eseményére.
     *
     * @param l Feliratkozó objektum
     */
    public final void addGraphListener(GraphListener l){
        listeners.add(l);
    }
    /**
     * Leiratkozás a gráf megváltozásának eseményéről.
     *
     * @param l Leiratkozó objektum
     */
    public final void removeGraphListener(GraphListener l){
        listeners.remove(l);
    }

    /**
     * Gráf megváltozás esemény kiváltása a gráf elemeinek módosítása után.
     */
    public final void changed(){
        for(GraphElement g: changes){
            g.changed();
        }
        changes.clear();
        GraphEvent evt = new GraphEvent(this);
        for(GraphListener l: listeners){
            l.changed(evt);
        }
    }
    /**
     * Gráf megváltozás esemény animált kiváltása a gráf elemeinek módosítása
     * után, paramétere az animáció állása.
     *
     * @param percent Animáció állása
     */
    public final void animation(float percent){
        GraphEvent evt = new GraphEvent(this, percent);
        for(GraphListener l: listeners){
            l.animation(evt);
        }
    }

    /**
     * A gráf valódi szélessége rajzoláskor. Értéke a leszármazott osztályokban
     * a belső <i>drawerWidth</i>, <i>drawerBorderX</i> változók segítségével
     * adható meg.
     *
     * @return Szélesség
     */
    public final int getDrawerWidth(){
        return drawerWidth + 2*drawerBorderX;
    }
    /**
     * A gráf valódi magassága rajzoláskor. Értéke a leszármazott osztályokban
     * a belső <i>drawerHeight</i>, <i>drawerBorderY</i> változók segítségével
     * adható meg.
     *
     * @return Magassága
     */
    public final int getDrawerHeight(){
        return drawerHeight + 2*drawerBorderY;
    }
    /**
     * Vízszintes szegély vastagsága rajzoláskor. Értéke a leszármazott
     * osztályokban a belső <i>drawerBorderX</i> változó segítségével adható
     * meg.
     *
     * @return Vízszintes szegély
     */
    public final int getDrawerBorderX(){
        return drawerBorderX;
    }
    /**
     * Függőleges szegély vastagsága rajzoláskor. Értéke a leszármazott
     * osztályokban a belső <i>drawerBorderY</i> változó segítségével adható
     * meg.
     *
     * @return Függőleges szegély
     */
    public final int getDrawerBorderY(){
        return drawerBorderY;
    }
}
