package bogqaai.graph;

/**
 * A gráfot leíró osztály, az absztrakt gráf ősosztályból származik.
 * Tartalmazza a csúcsokat és éleket leíró osztályokat és egy saját rajzoló
 * osztályt. A csúcsokat és éleket egy a sorszámuk alapján rendezett tömbben
 * illetve mátrixban tárolja. A reprezentáció kiválasztásakor a megjelenítés és
 * a szemléltetés hatékonysága volt a meghatározó szempont, a gráf változtatási
 * műveletek hatékonyságának rovására. A tömbös ábrázolással a csúcsok
 * bejárásának és elérésének hatékonnyá tétele, a rendezettséggel az
 * algoritmusok determinisztikussá tétele volt a cél. Az osztály az ősosztály
 * funkcionalitása mellett metódusokat biztosít a gráf mentésére és
 * beolvasására, változtatási és kirajzoló műveleteire.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class Graph extends AbstractGraph {
    /**
     * A gráf csúcsait leíró osztály, az absztrakt gráf elem ősosztályból
     * származik. Az ősosztály működése mellett minden csúcs rendelkezik egy, a
     * felhasználó által megadott, a gráfon belül egyedi sorszámmal, tárolja
     * a koordinátáit és a csúcsok tömbjében lévő helyét. A csúcs törölhető,
     * sorszáma változtatható és összehasonlítható önmagával a rendezett
     * tároláshoz. Kirajzoláskor felirata a sorszáma.
     */
    public class Vertex extends GraphElement implements Comparable<Vertex> {
        /** Sorszám. */
        private int id;
        /** A csúcsok listájában lévő hely. */
        private int ord;
        /** X koordináta. */
        private int x = 0;
        /** Y koordináta. */
        private int y = 0;

        /**
         * Csúcs belső konstruktora, paramétere a sorszám, és a helye a csúcsok
         * tömbjében.
         *
         * @param id
         * @param ord
         */
        private Vertex(int id, int ord){
            this.id = id;
            this.ord = ord;
        }

        /**
         * A csúcsok tömbjében lévő hely lekérdezése.
         *
         * @return A csúcsok tömbjében lévő hely
         */
        public int getOrd(){
            return ord;
        }
        /**
         * Sorszám lekérdezése.
         *
         * @return Sorszám
         */
        public int getId(){
            return id;
        }
        /**
         * Sorszám megváltoztatása, paramétere az új sorszám. Visszatérési
         * értéke jelzi a művelet sikerességét.
         *
         * @param id Új sorszám
         * @return Sikeresség
         */
        public boolean setId(int id){
            return setVertexId(this, id);
        }
        /**
         * X koordináta lekérdezése.
         *
         * @return X koordináta
         */
        public int getX(){
            return x;
        }
        /**
         * Y koordináta lekérdezése.
         *
         * @return Y koordináta
         */
        public int getY(){
            return y;
        }
        /**
         * Koordináták beállítása, opcionális rácshoz igazítással. A művelet
         * egy gráf megváltozási eseményt generál.
         *
         * @param x X koordináta
         * @param y Y koordináta
         * @param aligned Igazított-e
         */
        public void setPos(int x, int y, boolean aligned){
            if(aligned){
                x = Math.round((float)x/GraphDrawer.RADIUS) * GraphDrawer.RADIUS;
                y = Math.round((float)y/GraphDrawer.RADIUS) * GraphDrawer.RADIUS;
            }
            this.x = x < 0 ? 0 : x;
            this.y = y < 0 ? 0 : y;
            internalDimensionChanged();
            Graph.this.changed();
        }

        /**
         * Csúcs törlése.
         *
         * @return Sikeresség
         */
        public boolean remove(){
            return removeVertex(this);
        }

        /**
         * Csúcsok azonosságának vizsgálata.
         *
         * @param obj Másik csúcs
         * @return Azonosak-e
         */
        @Override
        public boolean equals(Object obj){
            if(obj instanceof Vertex){
                return ((Vertex)obj).id == id;
            }
            return false;
        }
        /**
         * Hasítókód a sorszám alapján.
         *
         * @return Hasítókód
         */
        @Override
        public int hashCode() {
            return id;
        }
        /**
         * Összehasonlító operátor a rendezett tároláshoz: a sorszámuk szerint
         * hasonlítja össze a csúcsokat.
         *
         * @param o Másik csúcs
         * @return Összehasonlítás
         */
        @Override
        public int compareTo(Vertex o){
            return Integer.valueOf(id).compareTo(Integer.valueOf(o.id));
        }

        /**
         * A sorszám feliratként.
         *
         * @return Felirat
         */
        @Override
        public String toString(){
            return "" + id;
        }
    }

    /**
     * A gráf csúcsait leíró osztály, az absztrakt gráf elem ősosztályból
     * származik. Az ősosztály működése mellett minden él rendelkezik egy
     * súllyal és ismeri a két végpontját. Az él törölhető és a súlya
     * változtatható. Kirajzoláskor felirata a súlya.
     */
    public class Edge extends GraphElement {
        /** Súly. */
        private int weight;
        /** Egyik végpont. */
        private final Vertex u;
        /** Másik végpont. */
        private final Vertex v;

        /**
         * Él belső konstruktora, paramétere a súlya és a végpontjai.
         *
         * @param u Egyik végpont
         * @param v Másik végpont
         * @param weight Súly
         */
        private Edge(Vertex u, Vertex v, int weight){
            this.u = u;
            this.v = v;
            this.weight = weight;
        }

        /**
         * Súly lekérdezése.
         *
         * @return Súly
         */
        public int getWeight(){
            return weight;
        }
        /**
         * Súly megváltoztatása. A művelet egy gráf megváltozási eseményt
         * generál.
         *
         * @param weight Új súly
         */
        public void setWeight(int weight){
            this.weight = weight;
            Graph.this.changed();
        }
        /**
         * Kisebbik sorszámú végpont lekérdezése.
         *
         * @return Kisebbik végpont
         */
        public Vertex getVertexU(){
            return u.id < v.id ? u : v;
        }
        /**
         * Nagyobbik sorszámú végpont lekérdezése.
         *
         * @return Nagyobbik végpont
         */
        public Vertex getVertexV(){
            return u.id < v.id ? v : u;
        }

        /**
         * Él törlése.
         *
         * @return Sikeresség
         */
        public boolean remove(){
            return removeEdge(this);
        }

        /**
         * A súly feliratként.
         *
         * @return Felirat
         */
        @Override
        public String toString(){
            return "" + weight;
        }
    }

    /**
     * A csúcsokat és éleket kirajzoló osztály, a gráf elemeit kirajzoló
     * absztrakt osztály leszármazottja. Definiálja a rajzolás paramétereit.
     */
    private class GraphDrawer extends Drawer {
        /** A csúcsokat megjelenítő kör sugara. */
        public static final int RADIUS = 20;

        /**
         * A rajzoló osztály konstruktora, paramétere a rajzolási cél, és a
         * szegély használata. Beállítja a rajzolás paramétereit.
         *
         * @param g A rajzolási cél
         * @param border Szegély használata
         */
        private GraphDrawer(java.awt.Graphics2D g, boolean border){
            super(g, border, RADIUS, RADIUS, false, 20, 14, 1, 2);
        }
    }

    /** Üres fájl esetén megjelenített hibaüzenet. */
    private final String emptyMessage;
    /** Fájlban szereplő hibás érték esetén megjelenített hibaüzenet. */
    private final String intInvalidMessage;
    /** Hiányzó csúcs definíció üzenete. */
    private final String vertexMissingMessage;
    /** Hibás csúcs definíció üzenete. */
    private final String vertexInvalidMessage;
    /** Ismétlődő csúcs definíció üzenete. */
    private final String vertexFailMessage;
    /** Hibás él definíció üzenete. */
    private final String edgeInvalidMessage;
    /** Él definícióban hibás csúcs megadás üzenete. */
    private final String edgeFailMessage;

    /** Csúcsokat tároló rendezett tömb. */
    private final java.util.ArrayList<Vertex> vertices = new java.util.ArrayList<Vertex>();
    /** Éleket tároló, a végpontjaik szerint rendezett mátrix. */
    private final java.util.ArrayList<java.util.ArrayList<Edge>> edges = new java.util.ArrayList<java.util.ArrayList<Edge>>();
    /** Csúcsok száma. */
    private int size = 0;
    /** A szerkeszthetőség letiltása. */
    private boolean locked = false;

    /**
     * A gráf osztály konstruktora. Betölti a fájl megnyitással kapcsolatos
     * hibaüzeneteket a <i>bogqaai/graph/Graph.properties</i> fájlból.
     */
    public Graph(){
        drawerBorderX = drawerBorderY = Drawer.BORDER + GraphDrawer.RADIUS;
        // erőforrások
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/graph/Graph");
        emptyMessage = bundle.getString("Graph.empty.message");
        intInvalidMessage = bundle.getString("Graph.int.invalid.message");
        vertexMissingMessage = bundle.getString("Graph.vertex.missing.message");
        vertexInvalidMessage = bundle.getString("Graph.vertex.invalid.message");
        vertexFailMessage = bundle.getString("Graph.vertex.fail.message");
        edgeInvalidMessage = bundle.getString("Graph.edge.invalid.message");
        edgeFailMessage = bundle.getString("Graph.edge.fail.message");
    }

    /**
     * Gráf megnyitása fájlból. Törli a gráf tartalmát és beállításait és
     * betölti a gráfot tartalmazó fájlt. Sikertelen megnyitás esetén
     * <i>java.io.IOException</i>, hibás fájl formátum esetén
     * <i>GraphFormatException</i> kivételt generál a hiba okát megjelölve.
     *
     * @param in
     * @throws java.io.IOException Fájl művelet hibája
     * @throws GraphFormatException Hibás fájl formátum
     */
    public void open(java.io.Reader in) throws java.io.IOException {
        boolean fail = true;
        clear();
        try{
            java.io.LineNumberReader br = new java.io.LineNumberReader(in);
            // gráf
            String line;
            if((line = br.readLine()) == null) throw new GraphFormatException(emptyMessage);
            int n = internalParseInt(line.trim(), br.getLineNumber());
            if(n < 0) throw new GraphFormatException(intInvalidMessage, "" + br.getLineNumber(), "" + n);
            vertices.ensureCapacity(n);
            edges.addAll(internalCreateList(n, (java.util.ArrayList<Edge>)null));
            for(int i=0; i<n; ++i){
                edges.set(i, internalCreateList(n, (Edge)null));
            }
            // csúcsok
            for(int i=0; i<n; ++i){
                if((line = br.readLine()) == null) throw new GraphFormatException(vertexMissingMessage, "" + br.getLineNumber());
                String[] pieces = line.trim().split("\\s");
                if(pieces.length != 3) throw new GraphFormatException(vertexInvalidMessage, "" + br.getLineNumber());
                int id = internalParseInt(pieces[0], br.getLineNumber());
                int index = internalGetVertexIndex(id);
                if(index >= 0) throw new GraphFormatException(vertexFailMessage, "" + br.getLineNumber());
                int x = internalParseInt(pieces[1], br.getLineNumber());
                int y = internalParseInt(pieces[2], br.getLineNumber());
                internalAddVertex(id, -index-1, x, y, false);
            }
            // élek
            while((line = br.readLine()) != null){
                String[] pieces = line.trim().split("\\s");
                if(pieces.length != 3) throw new GraphFormatException(edgeInvalidMessage, "" + br.getLineNumber());
                Vertex u = internalGetVertex(internalParseInt(pieces[0], br.getLineNumber()));
                Vertex v = internalGetVertex(internalParseInt(pieces[1], br.getLineNumber()));
                int w = internalParseInt(pieces[2], br.getLineNumber());
                if(u==null || v==null || u.id==v.id) throw new GraphFormatException(edgeFailMessage, "" + br.getLineNumber());
                internalAddEdge(u, v, w);
            }
            internalDimensionChanged();
            changed();
            fail = false;
        }finally{
            if(fail){
                clear();
            }
        }
    }
    /**
     * Gráf mentése fájlba. Sikertelen megnyitás esetén
     * <i>java.io.IOException</i> kivételt generál a hiba okát megjelölve.
     *
     * @param out
     * @throws java.io.IOException
     */
    public void save(java.io.Writer out) throws java.io.IOException {
        java.io.PrintWriter pw = new java.io.PrintWriter(out);
        pw.println(size);
        // csúcsok
        for(int i=0; i<size; ++i){
            Vertex u = vertices.get(i);
            pw.println("" + u.id + " " + u.x + " " + u.y);
        }
        // élek
        for(int i=0; i<size; ++i){
            Vertex u = vertices.get(i);
            for(int j=i+1; j<size; ++j){
                Vertex v = vertices.get(j);
                Edge e = getEdge(u, v);
                if(e != null){
                    pw.println("" + u.id + " " + v.id + " " + e.weight);
                }
            }
        }
    }

    /**
     * Gráf tartalmának és beállításainak törlése.
     */
    @Override
    protected void internalClear(){
        locked = false;
        size = 0;
        vertices.clear();
        edges.clear();
        drawerWidth = 0;
        drawerHeight = 0;
    }

    /**
     * Szöveg számmá alakítása fájl beolvasásakor, belső metódus.
     *
     * @param s Szöveg
     * @param line Sor
     * @return Szám
     */
    private int internalParseInt(String s, int line){
        try{
            return Integer.parseInt(s);
        }catch(NumberFormatException e){
            throw new GraphFormatException(intInvalidMessage, "" + line, s);
        }
    }
    /**
     * Tömb létrehozása és feltöltése, belső metódus.
     *
     * @param <T> Elemtípus
     * @param newSize Méret
     * @param o Feltöltendő elem
     * @return Létrehozott tömb
     */
    private <T> java.util.ArrayList<T> internalCreateList(int newSize, T o){
        return new java.util.ArrayList<T>(java.util.Collections.nCopies(newSize, o));
    }
    /**
     * Elem mozgatása a tömbben, belső metódus.
     *
     * @param <T> Elemtípus
     * @param list Tömb
     * @param oldIndex Elem régi indexe
     * @param newIndex Elem új indexe
     */
    private <T> void internalMove(java.util.ArrayList<T> list, int oldIndex, int newIndex){
        T t = list.remove(oldIndex);
        list.add(newIndex, t);
    }
    /**
     * Csúcs helyének megkeresése a csúcsok rendezett tömbjében, bináris
     * kereséssel, belső metódus.
     *
     * @param id Keresendő sorszám
     * @return Csúcs helye a tömbben
     */
    private int internalGetVertexIndex(int id){
        return java.util.Collections.binarySearch(vertices, new Vertex(id, 0));
    }
    /**
     * Csúcs megkeresése a csúcsok rendezett tömbjében, bináris kereséssel,
     * belső metódus.
     *
     * @param id Keresendő sorszám
     * @return Csúcs, vagy <i>null</i>
     */
    private Vertex internalGetVertex(int id){
        int index = internalGetVertexIndex(id);
        if(index >= 0){
            return vertices.get(index);
        }
        return null;
    }
    /**
     * Csúcs létrehozása és hozzáadása a csúcsok tömbjéhez, belső metódus.
     *
     * @param id Sorszám
     * @param index Hely a csúcsok tömbjében
     * @param x X koordináta
     * @param y Y koordináta
     * @param aligned Igazított-e
     * @return Létrehozott csúcs
     */
    private Vertex internalAddVertex(int id, int index, int x, int y, boolean aligned){
        Vertex u = new Vertex(id, index);
        ++size;
        vertices.add(index, u);
        for(int i=index+1; i<size; ++i) vertices.get(i).ord = i;
        u.setPos(x, y, aligned);
        return u;
    }
    /**
     * Él létrehozása, belső metódus.
     *
     * @param u Egyik végpont
     * @param v Másik végpont
     * @param w Súly
     * @return Létrehozott él
     */
    private Edge internalAddEdge(Vertex u, Vertex v, int w){
        Edge e = new Edge(u, v, w);
        edges.get(u.ord).set(v.ord, e);
        edges.get(v.ord).set(u.ord, e);
        return e;
    }
    /**
     * Gráf méretének megváltozása, belső metódus.
     */
    private void internalDimensionChanged(){
        drawerWidth = 0;
        drawerHeight = 0;
        for(int i=0; i<size; ++i){
            Vertex u = vertices.get(i);
            if(u.x > drawerWidth) drawerWidth = u.x;
            if(u.y > drawerHeight) drawerHeight = u.y;
        }
    }

    /**
     * A gráf méretének, a csúcsok számának lekérdezése.
     *
     * @return Csúcsok száma
     */
    public int getSize(){
        return size;
    }
    /**
     * A csúcsok rendezett tömbjének lekérdezése.
     *
     * @return Csúcsok tömbje
     */
    public java.util.List<Vertex> getVertices(){
        return java.util.Collections.unmodifiableList(vertices);
    }
    /**
     * Két csúcs között húzódó él lekérdezése. Ha nincs ilyen él, akkor
     * visszatérési értéke <i>null</i>.
     *
     * @param u Egyik végpont
     * @param v Másik végpont
     * @return Él vagy <i>null</i>
     */
    public Edge getEdge(Vertex u, Vertex v){
        return edges.get(u.ord).get(v.ord);
    }
    /**
     * Csúcs keresése koordináták alapján. Visszatérési értéke az a legnagyobb
     * sorszámú csúcs, aminek a megjelenített körébe beleesnek a megadott
     * koordináták, vagy <i>null</i>, ha nincs ilyen.
     *
     * @param x X koordináta
     * @param y Y koordináta
     * @return Talált csúcs vagy <i>null</i>
     */
    public Vertex findVertex(int x, int y){
        for(int i=size-1; i>=0; --i){
            Vertex u = vertices.get(i);
            int dx = u.x - x;
            int dy = u.y - y;
            if(Math.sqrt(dx*dx + dy*dy) <= GraphDrawer.RADIUS){
                return u;
            }
        }
        return null;
    }

    /**
     * A gráf szerkesztésénék letiltása vagy engedélyezése.
     *
     * @param lock Letiltás
     */
    public void setLocked(boolean lock){
        locked = lock;
    }
    /**
     * A szerkeszthetőség letiltásának lekérdezése.
     *
     * @return Letiltás
     */
    public boolean isLocked(){
        return locked;
    }

    /**
     * Csúcs hozzáadása a gráfhoz, megadott koordinátákon, opcionális rácshoz
     * igazítással. A létrehozott csúcs sorszáma a legnagyobb sorszámú csúcs
     * sorszámánál eggyel nagyobb, vagy 1, ha a gráf üres. Visszatérési értéke
     * a létrehozott csúcs, vagy <i>null</i>, ha a gráf nem szerkeszthető.
     *
     * @param x X koordináta
     * @param y Y koordináta
     * @param aligned Igazított-e
     * @return Létrehozott csúcs, vagy <i>null</i>
     */
    public Vertex addVertex(int x, int y, boolean aligned){
        if(!locked){
            int id = size==0 ? 1 : vertices.get(size-1).id + 1;
            for(int i=0; i<size; ++i){
                edges.get(i).add(null);
            }
            edges.add(internalCreateList(size+1, (Edge)null));
            return internalAddVertex(id, size, x, y, aligned);
        }
        return null;
    }
    /**
     * Él létrehozása vagy meglévő él kijelölése két csúcs között. Visszatérési
     * értéke az él, vagy <i>null</i>, ha a gráf nem szerkeszthető.
     *
     * @param u Egyik végpont
     * @param v Másik végpont
     * @return Él, vagy <i>null</i>
     */
    public Edge addEdge(Vertex u, Vertex v){
        if(!locked && u.id!=v.id){
            Edge e = getEdge(u, v);
            if(e == null) e = internalAddEdge(u, v, 1);
            return e;
        }
        return null;
    }
    /**
     * Csúcs törlése, ha a gráf szerkeszthető.
     *
     * @param u Törlendő csúcs
     * @return Sikeresség
     */
    public boolean removeVertex(Vertex u){
        if(!locked){
            --size;
            edges.remove(u.ord);
            for(int i=0; i<size; ++i){
                edges.get(i).remove(u.ord);
            }
            vertices.remove(u.ord);
            for(int i=u.ord; i<size; ++i){
                vertices.get(i).ord = i;
            }
            return true;
        }
        return false;
    }
    /**
     * Él törlése, ha a gráf szerkeszthető.
     *
     * @param e Törlendő él
     * @return Sikeresség
     */
    public boolean removeEdge(Edge e){
        if(!locked){
            edges.get(e.u.ord).set(e.v.ord, null);
            edges.get(e.v.ord).set(e.u.ord, null);
            return true;
        }
        return false;
    }
    /**
     * Csúcs sorszámának módosítása. Visszatérési értéke a művelet sikeressége.
     *
     * @param u Csúcs
     * @param id Új sorszám
     * @return Sikeresség
     */
    public boolean setVertexId(Vertex u, int id){
        if(!locked){
            if(u.id != id){
                int index = internalGetVertexIndex(id);
                if(index < 0){
                    index = -index-1;
                    if(index > u.ord) --index;
                    internalMove(vertices, u.ord, index);
                    internalMove(edges, u.ord, index);
                    for(int i=0; i<size; ++i){
                        internalMove(edges.get(i), u.ord, index);
                    }
                    int max = Math.max(index, u.ord);
                    for(int i=Math.min(index, u.ord); i<=max; ++i){
                        vertices.get(i).ord = i;
                    }
                    u.id = id;
                    changed();
                    return true;
                }
                return false;
            }
            return true;
        }
        return false;
    }

    /**
     * Összes csúcs és él szürkére vagy feketére színezése.
     *
     * @param gray Szürke vagy fekete
     */
    public void reset(boolean gray){
        resetVertices(gray);
        resetEdges(gray);
    }
    /**
     * Összes csúcs szürkére vagy feketére színezése.
     *
     * @param gray Szürke vagy fekete
     */
    public void resetVertices(boolean gray){
        for(int i=0; i<size; ++i){
            vertices.get(i).set(gray ? Color.GRAY : 0, 0);
        }
    }
    /**
     * Összes él szürkére vagy feketére színezése.
     *
     * @param gray Szürke vagy fekete
     */
    public void resetEdges(boolean gray){
        for(int i=0; i<size; ++i){
            for(int j=i+1; j<size; ++j){
                Edge e = edges.get(i).get(j);
                if(e != null){
                    e.set(gray ? Color.GRAY : 0, 0);
                }
            }
        }
    }

    /**
     * Rácspontok rajzolása, megadott szélességben és magasságban. Visszatérési
     * értéke a rajzoló objektum.
     *
     * @param g A rajzolási cél
     * @param width Szélesség
     * @param height Magasság
     * @return A rajzoló objektum
     */
    public Drawer drawGrid(java.awt.Graphics2D g, int width, int height){
        Drawer drawer = new GraphDrawer(g, true);
        drawer.drawGrid(width, height);
        return drawer;
    }
    /**
     * Gráf kirajzolása, paraméterei az élsúlyok megjelenítése és az animáció
     * állása. Visszatérési értéke a rajzoló objektum.
     *
     * @param g A rajzolási cél
     * @param weighted Élsúlyok megjelenítése
     * @param percent Animáció állása
     * @return A rajzoló objektum
     */
    public Drawer drawGraph(java.awt.Graphics2D g, boolean weighted, float percent){
        Drawer drawer = new GraphDrawer(g, true);
        for(int i=0; i<size; ++i){
            Vertex u = vertices.get(i);
            for(int j=i+1; j<size; ++j){
                Vertex v = vertices.get(j);
                Edge e = getEdge(u, v);
                if(e != null){
                    e.drawEdge(drawer, u.x, u.y, v.x, v.y, weighted, percent);
                }
            }
        }
        for(Vertex u: vertices){
            u.drawVertex(drawer, u.x, u.y, percent);
        }
        return drawer;
    }
    /**
     * Kijelölt csúcs kirajzolása egy megadott helyre.
     *
     * @param g A rajzolási cél
     * @param u Kijelölt csúcs
     * @param x X koordináta
     * @param y Y koordináta
     * @return A rajzoló objektum
     */
    public Drawer drawVertex(java.awt.Graphics2D g, Vertex u, int x, int y){
        Drawer drawer = new GraphDrawer(g, false);
        u.drawVertex(drawer, x, y);
        return drawer;
    }
    /**
     * Kijelölt él kirajzolása egy megadott helyre.
     *
     * @param g A rajzolási cél
     * @param e Kijelölt él
     * @param x1 Kezdőpont X koordináta
     * @param y1 Kezdőpont Y koordináta
     * @param x2 Végpont X koordináta
     * @param y2 Végpont Y koordináta
     * @param weighted Élsúlyok megjelenítése
     * @return A rajzoló objektum
     */
    public Drawer drawEdge(java.awt.Graphics2D g, Edge e, int x1, int y1, int x2, int y2, boolean weighted){
        Drawer drawer = new GraphDrawer(g, false);
        Vertex u = e.getVertexU();
        Vertex v = e.getVertexV();
        e.drawEdge(drawer, x1, y1, x2, y2, weighted);
        u.drawVertex(drawer, x1, y1);
        v.drawVertex(drawer, x2, y2);
        return drawer;
    }
}
