package bogqaai.graph.algorithm;

import bogqaai.graph.*;

/**
 * Az absztrakt algoritmus ősosztály. Az osztály egy alapértelmezett működési
 * mechanizmust biztosít az algoritmusoknak. Az algoritmus adatai lekérdezhetők,
 * az algoritmus léptethető, a háttérben lefuttatható, lekérdezhető az aktuális
 * lépés és az eredmény leírása, hozzákapcsolható adatszerkezet és kiválthat
 * eseményeket a gráf megváltozásáról és animálásáról. A tényleges működést a
 * leszármazott osztályok belső metódusok felülírásával és adattagok
 * módosításával tudják meghatározni.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public abstract class Algorithm {
    /** Ismeretlen eredmény, az algoritmus még nem fejezte be futását. */
    public static final int UNKNOWN = -1;
    /** Sikertelen futás. */
    public static final int FAIL = 0;
    /** Sikeres futás. */
    public static final int SUCCESS = 1;

    /**
     * Az algoritmusok adatait leíró rekord. Tartalmazza az algoritmus és a
     * kapcsolódó adatszerkezet nevét, az algoritmus rövid és részletes
     * leírását.
     */
    public static class Strings{
        /** Algoritmus neve. */
        public String title;
        /** Algoritmus leírása. */
        public String description;
        /** Adatszerkezet neve. */
        public String data;
        /** Részletes leírás, a súgó számára. */
        public String help;
    }

    /** Algoritmusok feliratait tartalmazó erőforrásfájl. */
    protected static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("bogqaai/graph/algorithm/Algorithm");
    /** Inicializáló lépés üzenet formátuma. */
    private static final String initFormat = bundle.getString("Algorithm.init");
    /** Lépések üzenet formátuma. */
    private static final String stepFormat = bundle.getString("Algorithm.step");
    /** Eredmény üzenet formátuma. */
    private static final String resultFormat = bundle.getString("Algorithm.result");

    /** Vizsgált gráf. */
    protected final Graph graph;
    /** Adatszerkezet. */
    protected AbstractData data = null;
    /** Futás eredménye. */
    protected int result = UNKNOWN;
    /** Aktuális lépés. */
    private int step = 0;
    /** Aktuális lépéshez tartozó szöveg. */
    private String text = "";

    /**
     * Az algoritmus osztály konstruktora, paramétere a vizsgálta gráf.
     * Létrehozásakor zárolja a gráfot, letiltja annak szerkeszthetőségét.
     * Az algoritmusok leírásait a
     * <i>bogqaai/graph/algorithm/Algorithm.properties</i> fájlból tölti be.
     *
     * @param graph Vizsgált gráf
     */
    public Algorithm(Graph graph){
        this.graph = graph;
        graph.setLocked(true);
    }

    /**
     * Élsúlyok használata, a leszármazott osztályokban definiált metódus.
     *
     * @return Élsúly használata
     */
    public abstract boolean isGraphWeighted();

    /**
     * Az algoritmus alaphelyzetbe állítása a leszármazott osztályokban.
     * Visszatérési értéke az inicializálás leírása.
     *
     * @return Inicializálás leírása
     */
    protected abstract String internalReset();
    /**
     * Az algoritmus léptetése a leszármazott osztályokban.
     * Visszatérési értéke a lépés szövege vagy az algoritmus eredménye.
     *
     * @return Lépés szövege vagy eredmény
     */
    protected abstract String internalStep();

    /**
     * Az algoritmus alaphelyzetbe állítása.
     */
    public final void reset(){
        graph.reset(true);
        result = UNKNOWN;
        step = 0;
        text = java.text.MessageFormat.format(initFormat, internalReset());
        changed();
    }
    /**
     * Az algoritmus léptetése, ha még nem fejezte be futását, lépés leírásának
     * formázott megjelenítése.
     */
    public final void step(){
        if(!isReady()){
            ++step;
            String stepText = internalStep();
            if(isReady()){
                text = java.text.MessageFormat.format(resultFormat, stepText);
            }else{
                text = java.text.MessageFormat.format(stepFormat, step, stepText);
            }
        }
    }
    /**
     * Az algoritmus futtatása a háttérben.
     */
    public final void run(){
        reset();
        if(!isReady()){
            String stepText = null;
            while(!isReady()){
                stepText = internalStep();
            }
            text = java.text.MessageFormat.format(resultFormat, stepText);
        }
        changed();
    }

    /**
     * Gráf megváltozás esemény kiváltása.
     */
    public final void changed(){
        graph.changed();
        if(data != null) data.changed();
    }
    /**
     * Gráf megváltozás esemény animált kiváltása.
     *
     * @param percent
     */
    public final void animation(float percent){
        graph.animation(percent);
    }

    /**
     * Az adatszerkezet lekérdezése, értékét a leszármazottak a belső
     * <i>data</i> adattagon keresztül állítják be. Visszatérési értéke
     * <i>null</i>, ha az algoritmus nem használja.
     *
     * @return Adatszerkezet vagy <i>null</i>
     */
    public final AbstractData getData(){
        return data;
    }

    /**
     * Megvizsgálja, hogy az algoritmus befejezte-e futását.
     *
     * @return Kész van-e
     */
    public final boolean isReady(){
        return result != UNKNOWN;
    }
    /**
     * Lekérdezi az algoritmus futásának eredményét. Ha az algoritmus még nem
     * ért véget, akkor értéke <i>UNKNOWN</i>.
     *
     * @return Futás eredménye
     */
    public final int getResult(){
        return result;
    }
    /**
     * Inicializáláshoz, lépéshez, eredményhez tartozó szöveg lekérdezése.
     *
     * @return Inicializálás, lépés, eredmény
     */
    public final String getText(){
        return text;
    }

    /**
     * Leszármazott osztály adatait tartalmazó rekord elkészítése.
     *
     * @param <T> Leszármazott osztály típusa
     * @param clazz Leszármazott osztály típusaváltozója
     * @return Algoritmus adatainak rekordja
     */
    public static <T extends Algorithm> Strings getStrings(Class<T> clazz){
        Strings s = new Strings();
        s.title = bundle.getString(clazz.getSimpleName() + ".title");
        s.description = bundle.getString(clazz.getSimpleName() + ".description");
        s.data = bundle.getString(clazz.getSimpleName() + ".data");
        s.help = bundle.getString(clazz.getSimpleName() + ".help");
        return s;
    }
}
