package bogqaai.graph.algorithm;

import bogqaai.graph.*;

/**
 * A párosság vizsgálatát megvalósító algoritmus osztálya, őse az absztrakt
 * algoritmus osztály.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class Bipartite extends Algorithm {
    /** Piros csúcsok színe. */
    public static final int COLOR0 = Graph.Color.RED;
    /** Kék csúcsok színe. */
    public static final int COLOR1 = Graph.Color.BLUE;
    /** Csúcsok színei. */
    private static final int[] COLORS = {
        COLOR0, COLOR1
    };

    /** Inicializáló lépés. */
    private static final String initMessage = bundle.getString("Bipartite.init.message");
    /** Sikeres lefutás eredménye. */
    private static final String successMessage = bundle.getString("Bipartite.success.message");
    /** Sikertelen lefutás eredménye. */
    private static final String failMessage = bundle.getString("Bipartite.fail.message");
    /** Bejárási lépés leírása. */
    private static final String stepMessage =  bundle.getString("Bipartite.step.message");
    /** Komponens felderítés lépés leírása. */
    private static final String stepComponentMessage = bundle.getString("Bipartite.stepcomponent.message");

    /** Sor adatszerkezet. */
    private final Queue queue;
    /** Aktuálisan vizsgált csúcs sorszáma. */
    private int vertex = 0;
    /** Előzőleg vizsgált csúcs. */
    private Graph.Vertex last = null;

    /**
     * Párosság vizsgálata algoritmus konstruktora, paramétere a vizsgált gráf.
     *
     * @param graph Vizsgált gráf
     */
    public Bipartite(Graph graph){
        super(graph);
        queue = new Queue(graph);
        data = queue;
    }

    /**
     * Az algoritmus nem használ élsúlyokat, visszatérési értéke hamis.
     *
     * @return Hamis
     */
    @Override
    public boolean isGraphWeighted(){
        return false;
    }

    /** Az algoritmus alaphelyzetbe állítása. */
    @Override
    protected String internalReset(){
        queue.reset();
        vertex = 0;
        last = null;
        return initMessage;
    }

    /** Az algoritmus léptetése. */
    @Override
    protected String internalStep(){
        String resultText;
        if(last != null){
            last.set(last.getColor(), 0);
        }
        if(!queue.isEmpty()){
            // bejárási lépés
            Graph.Vertex u = queue.pop();
            resultText = java.text.MessageFormat.format(stepMessage, u.toString());
            int color = u.getColor() == COLOR0 ? 1 : 0;
            for(Graph.Vertex v: graph.getVertices()){
                Graph.Edge e = graph.getEdge(u, v);
                if(e != null){
                    if(e.getColor() == Graph.Color.GRAY){
                        e.set(0, 0, e.getVertexU()==u ? Graph.Animation.LTR : Graph.Animation.RTL);
                    }
                    if(v.getColor() == Graph.Color.GRAY){
                        v.set(COLORS[color], 0, Graph.Animation.DELAYED);
                        queue.push(v);
                    }else if(v.getColor() != COLORS[color]){
                        // ütköző színű csúcsok
                        v.set(v.getColor(), Graph.Fill.MARKED, Graph.Animation.DELAYED);
                        resultText = java.text.MessageFormat.format(failMessage, u.toString(), v.toString());
                        result = FAIL;
                        break;
                    }
                }
            }
            u.set(u.getColor(), Graph.Fill.MARKED);
            last = u;
        }else{
            // komponens felderítés lépés
            queue.reset();
            java.util.List<Graph.Vertex> vertices = graph.getVertices();
            while(vertex < graph.getSize() && vertices.get(vertex).getColor() != Graph.Color.GRAY) ++vertex;
            if(vertex < graph.getSize()){
                Graph.Vertex u = vertices.get(vertex);
                resultText = java.text.MessageFormat.format(stepComponentMessage, u.toString());
                u.set(COLOR0, Graph.Fill.MARKED);
                queue.push(u);
                ++vertex;
                last = u;
            }else{
                resultText = successMessage;
                result = SUCCESS;
            }
        }
        return resultText;
    }
}
