package bogqaai.graph.algorithm;

import bogqaai.graph.*;

/**
 * Az Unió-Holvan adatszerkezetet megvalósító osztály, az absztrakt
 * adatszerkezet osztály leszármazottja. Az adatszerkezet az unió és a holvan
 * műveletekkel rendelkezik, kirajzolásnál a reprezentáló fastruktúrát jeleníti
 * meg. Az adatszerkezet implementálása a hatékonyabb megjelenítés érdekében egy
 * láncolt fa adatszerkezet segítségével történik, amelynek elemei a csúcsokhoz
 * tartozó részhalmazok. A fa elemei a szülőmutató mellett tárolják az első és
 * utolsó gyereküket, és a közvetlen szomszédjaikat is.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class DisjointSet extends AbstractData {
    /**
     * A csúcsokhoz tartozó részhalmazok, a fa elemeinek belső osztálya, az
     * absztrakt gráf elem osztály leszármazottja. Tárolja a halmaz méretét,
     * sorszámát és mutatóit.
     */
    private class Set extends GraphElement{
        /** Hozzárendelt csúcs. */
        private final Graph.Vertex vertex;

        /** Sorszám. */
        private final int id;
        /** Szülőmutató. */
        private Set parent = null;
        /** Halmaz mérete. */
        private int size = 1;

        /** Első gyerek a fában. */
        private Set firstChild = null;
        /** Utolsó gyerek a fában. */
        private Set lastChild = null;
        /** Következő szomszéd a fában. */
        private Set next = null;
        /** Megelőző szomszéd a fában. */
        private Set prev = null;

        /**
         * A halmaz osztály konstruktora, paramétere a hozzárendelt csúcs.
         *
         * @param vertex Hozzárendelt csúcs
         */
        private Set(Graph.Vertex vertex){
            this.vertex = vertex;
            this.id = vertex.getOrd();
        }

        /**
         * A halmaz felirata a hozzárendelt csúcs felirata.
         *
         * @return Felirat
         */
        @Override
        public String toString(){
            return vertex.toString();
        }
    }

    /** Vizsgált gráf. */
    private final Graph graph;
    /** Csúcsok halmazainak tömbje. */
    private final Set[] sets;
    /** Első halmaz. */
    private Set firstSet = null;

    /**
     * Az Unió-Holvan adatszerkezetet konstruktora, paramétere a vizsgált gráf.
     *
     * @param graph Vizsgált gráf
     */
    public DisjointSet(Graph graph){
        this.graph = graph;
        int size = graph.getSize();
        sets = new Set[size];
        drawerWidth = drawerHeight = size > 0 ? (size-1)*DataDrawer.DISTANCE : 0;
    }

    /**
     * Halmazok létrehozása, minden csúcs külön halmazba sorolása.
     */
    public void fill(){
        int size = graph.getSize();
        java.util.List<Graph.Vertex> vertices = graph.getVertices();
        for(int i=0; i<size; ++i){
            sets[i] = new Set(vertices.get(i));
        }
        // többirányú láncolás
        for(int i=0; i<size-1; ++i){
            sets[i].next = sets[i+1];
            sets[i+1].prev = sets[i];
        }
        firstSet = size > 0 ? sets[0] : null;
    }

    /**
     * Holvan belső művelete, meghatározza, hogy az adott csúcs részhalmaza
     * melyik halmazban van, a halmaz gyökérelemét megadva.
     *
     * @param s Részhalmaz
     * @return Halmaz
     */
    private Set internalFind(Set s){
        if(s.parent != null){
            Set parent = internalFind(s.parent);
            // útösszenyomás
            if(parent != s.parent){
                if(s.parent.firstChild == s){
                    s.parent.firstChild = s.next;
                }else{
                    s.prev.next = s.next;
                }
                if(s.parent.lastChild == s){
                    s.parent.lastChild = s.prev;
                }else{
                    s.next.prev = s.prev;
                }

                s.parent = parent;
                parent.lastChild.next = s;
                s.prev = parent.lastChild;
                parent.lastChild = s;
                s.next = null;
            }
            return parent;
        }
        return s;
    }
    /**
     * Holvan művelet, meghatározza, hogy az adott csúcs hányadik halmazban van.
     * A művelet a csúcsot tartalmazó fa gyökérelemének sorszámát adja meg, a
     * keresés közben útösszenyomást hajtva végre.
     *
     * @param u Csúcs
     * @return Halmaz sorszáma
     */
    public int find(Graph.Vertex u){
        return internalFind(sets[u.getOrd()]).id;
    }

    /**
     * Unió belső művelete, két gyökérelemmel adott halmazt egyesít.
     *
     * @param s1 Egyik halmaz
     * @param s2 Másik halmaz
     */
    private void internalUnion(Set s1, Set s2){
        // kisebb halmaz csatlakoztatása a nagyobb alá
        if(s2.size > s1.size){
            Set tmp = s1;
            s1 = s2;
            s2 = tmp;
        }

        // többszörös láncolás
        if(firstSet == s2){
            firstSet = s2.next;
        }else{
            s2.prev.next = s2.next;
        }
        if(s2.next != null){
            s2.next.prev = s2.prev;
        }

        s1.size += s2.size;
        s2.parent = s1;
        if(s1.lastChild != null){
            s1.lastChild.next = s2;
        }else{
            s1.firstChild = s2;
        }
        s2.prev = s1.lastChild;
        s1.lastChild = s2;
        s2.next = null;
    }
    /**
     * Unió művelet, két csúcshoz tartozó halmazt egyesít. A nagyobb halmazhoz
     * tartozó fa gyökeréhez gyerekként hozzákapcsolja a másikhoz tartozó fa
     * gyökerét.
     *
     * @param u
     * @param v
     */
    public void union(Graph.Vertex u, Graph.Vertex v){
        internalUnion(internalFind(sets[u.getOrd()]), internalFind(sets[v.getOrd()]));
    }

    /**
     * Kirajzolás belső, rekurzív művelete.
     *
     * @param drawer A rajzoló objektum
     * @param s Részhalmaz
     * @param x X koordináta
     * @param y Y koordináta
     * @return Következő X koordináta
     */
    private int internalDraw(Drawer drawer, Set s, int x, int y){
        int xx = x;
        if(s.firstChild != null){
            int y1 = y + DataDrawer.DISTANCE/2;
            int y2 = y + DataDrawer.DISTANCE;
            drawer.drawEdge(xx, y, xx, y1, null);
            // gyerekek részhalmazainak kirajzolása, X koordináta léptetése
            int x1 = xx;
            for(Set s1=s.firstChild; s1!=null; s1=s1.next){
                drawer.drawEdge(x1, y1, xx, y1, null);
                drawer.drawEdge(xx, y1, xx, y2, null);
                x1 = xx;
                xx = internalDraw(drawer, s1, xx, y2);
            }
        }else{
            xx += DataDrawer.DISTANCE;
        }
        s.drawVertex(drawer, x, y);
        return xx;
    }

    /**
     * Halmazok fastruktúrájának a kirajzolása.
     * Visszatérési értéke a rajzoló objektum.
     *
     * @param g A rajzolási cél
     * @return A rajzoló objektum
     */
    @Override
    public Drawer draw(java.awt.Graphics2D g){
        Drawer drawer = new DataDrawer(g);
        int x = 0, y = 0;
        for(Set s=firstSet; s!=null; s=s.next){
            x = internalDraw(drawer, s, x, y);
        }
        return drawer;
    }
}
