package bogqaai.graph.algorithm;

import bogqaai.graph.*;

/**
 * A kupac adatszerkezetet megvalósító osztály a gráf csúcsainak tárolására, az
 * absztrakt adatszerkezet osztály leszármazottja. A kupacban a csúcsok
 * rendezése elsődlegesen a csúcs <i>data</i> adattagjához hozzárendelt érték
 * alapján, másodlagosan a csúcs sorszáma alapján történik. A kupac rendelkezik
 * a szokásos műveletekkel - elem berakása, módosítása, legkisebb elem
 * kivétele -, kirajzolásnál megjeleníti a kupacból legutoljára kivett elemet
 * és a kupacban lévő elemeket a fastruktúrának megfelelően. A kupac
 * implementálása egy tömb segítségével történik, a tömb a fastruktúra szerint
 * szintfolytonosan tárolja a kupac elemeit.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class Heap extends AbstractData {
    /**
     * A kupac elemeinek belső osztálya, az absztrakt gráf elem osztály
     * leszármazottja. Az elemekhez a gráf csúcsait lehet hozzárendelni.
     */
    private class Item extends GraphElement implements Comparable<Item> {
        /** Hozzárendelt csúcs. */
        private final Graph.Vertex vertex;
        /** Elem helye a kupacot reprezentáló tömbben. */
        private int pos;
        /** X koordináta. */
        private int x;
        /** Y koordináta. */
        private int y;

        /**
         * A sor elem osztály konstruktora, paramétere a hozzárendelt csúcs, és
         * az elem koordinátái.
         *
         * @param vertex Hozzárendelt csúcs
         * @param x X koordináta
         * @param y Y koordináta
         */
        private Item(Graph.Vertex vertex, int x, int y){
            this.pos = vertex.getOrd();
            this.x = x;
            this.y = y;
            this.vertex = vertex;
        }

        /**
         * Összehasonlító operátor a rendezett tároláshoz: a hozzárendelt csúcs
         * <i>data</i> adattagja és sorszáma alapján.
         *
         * @param o Másik elem
         * @return Összehasonlítás
         */
        @Override
        @SuppressWarnings("unchecked")
        public int compareTo(Item o){
            int compare = 0;
            if(vertex.data instanceof Comparable<?>){
                compare = ((Comparable<Object>)vertex.data).compareTo(o.vertex.data);
            }
            if(compare == 0) compare = vertex.compareTo(o.vertex);
            return compare;
        }

        /**
         * Az elem felirata a hozzárendelt csúcs felirata és a csúcs <i>data</i>
         * adattagjának felirata, többsoros feliratként.
         *
         * @return Felirat
         */
        @Override
        public String toString(){
            return vertex.toString() + "\n" + vertex.data.toString();
        }
    }

    /** Vizsgált gráf. */
    private final Graph graph;
    /** Elemek tömbje sorszám szerint rendezve. */
    private final java.util.ArrayList<Item> items;
    /** A kupacot reprezentáló tömb. */
    private final java.util.ArrayList<Item> heap;
    /** Szélesség. */
    private final int width;
    /** Magasság. */
    private final int height;
    /** A kupacból utoljára kivett elem. */
    private Item last = null;

    /**
     * A kupac adatszerkezetet konstruktora, paramétere a vizsgált gráf.
     *
     * @param graph Vizsgált gráf
     */
    public Heap(Graph graph){
        this.graph = graph;
        int size = graph.getSize();
        height = (int)Math.ceil(Math.log(size+1)/Math.log(2)) + 1;
        width = 1 << (height-2);
        drawerHeight =  2*height*DataDrawer.DISTANCE;
        drawerWidth = width*DataDrawer.DISTANCE;
        drawerBorderX = Drawer.BORDER + DataDrawer.RADIUS;
        drawerBorderY = Drawer.BORDER + 2*DataDrawer.RADIUS;
        items = new java.util.ArrayList<Item>(size);
        heap = new java.util.ArrayList<Item>(size);
    }

    /**
     * Elem mozgatása megadott pozícióra, belső metódus.
     *
     * @param item Mozgatandó elem
     * @param pos Cél pozíció
     */
    private void moveTo(Item item, int pos){
        Item old = heap.get(pos);
        old.pos = item.pos;
        item.pos = pos;
        int t = item.x;
        item.x = old.x;
        old.x = t;
        t = item.y;
        item.y = old.y;
        old.y = t;
        heap.set(item.pos, item);
        heap.set(old.pos, old);
    }
    /**
     * Elem mozgatása lefelé, belső metódus.
     *
     * @param pos Mozgatandó elem helye a tömbben
     */
    private void moveDown(int pos){
        Item item = heap.get(pos);
        for(int child; (child = 2*item.pos+1) < heap.size(); ){
            if(child+1 < heap.size() && heap.get(child+1).compareTo(heap.get(child)) < 0){
                ++child;
            }
            if(item.compareTo(heap.get(child)) > 0){
                moveTo(item, child);
            }else{
                break;
            }
        }
    }
    /**
     * Elem mozgatása felfelé, belső metódus.
     *
     * @param pos Mozgatandó elem helye a tömbben
     */
    private void moveUp(int pos){
        Item item = heap.get(pos);
        while(item.pos > 0){
            int parent = (item.pos-1)/2;
            if(item.compareTo(heap.get(parent)) < 0){
                moveTo(item, parent);
            }else{
                break;
            }
        }
    }

    /**
     * Legkisebb elem kivétele a kupacból, kupac helyreállítása. Kivételkor
     * helyére a tömb utolsó eleme kerül, a kupachoz tartozó fa utolsó levele.
     * A kupac helyreállítása az elemek cseréivel, az első helyre került elem
     * fastruktúrabeli lefelé mozgatásával történik.
     *
     * @return Legkisebb elemhez hozzárendelt csúcs
     */
    public Graph.Vertex pop(){
        last = heap.get(0);
        items.set(last.vertex.getOrd(), null);
        if(heap.size() > 1){
            Item item = heap.remove(heap.size() - 1);
            item.pos = 0;
            item.x = width*DataDrawer.DISTANCE/2;
            item.y = 2*DataDrawer.DISTANCE;
            heap.set(0, item);
            moveDown(0);
        }else{
            heap.remove(0);
        }
        last.set(0, Graph.Fill.MARKED);
        return last.vertex;
    }
    /**
     * A kupac ürességének vizsgálata. Visszatérési értéke igaz, ha a kupac
     * üres.
     *
     * @return Üres-e
     */
    public boolean isEmpty(){
        return heap.isEmpty();
    }
    /**
     * Kupac alaphelyzetbe állítása, a tömbök és az utoljára kivett elemnek a
     * törlése.
     */
    public void reset(){
        last = null;
        items.clear();
        heap.clear();
    }
    /**
     * Kupac feltöltése a vizsgált gráf csúcsaival.
     */
    public void fill(){
        reset();
        int i=1;
        int j=1;
        int k=1;
        int l=width;
        for(Graph.Vertex u: graph.getVertices()){
            Item item = new Item(u, (width + (2*i-k-1)*l)*DataDrawer.DISTANCE/2, 2*j*DataDrawer.DISTANCE);
            items.add(item);
            heap.add(item);
            moveUp(item.pos);
            if(i < k){
                ++i;
            }else{
                i = 1;
                ++j;
                k *= 2;
                l /= 2;
            }
        }
    }

    /**
     * Elem megváltoztatása a kupacban, a kupac helyreállítása. Elem
     * megváltoztatása a kupacban, a kupac helyreállítása. A művelet
     * feltételezi, hogy a megváltoztatott elem értéke nem nőtt. A kupac
     * helyreállítása az elemek cseréivel, a megváltozott elem fastruktúrabeli
     * felfelé mozgatásával történik.
     *
     * @param u Megváltozott elemhez hozzárendelt csúcs
     */
    public void itemChanged(Graph.Vertex u){
        Item item = items.get(u.getOrd());
        moveUp(item.pos);
    }

    /**
     * Adatszerkezet beállításainak törlése.
     */
    @Override
    protected void internalClear(){
        reset();
    }

    /**
     * Kupac fastruktúra és a kupacból legutoljára kivett elem kirajzolása.
     * Visszatérési értéke a rajzoló objektum.
     *
     * @param g A rajzolási cél
     * @return A rajzoló objektum
     */
    @Override
    public Drawer draw(java.awt.Graphics2D g){
        Drawer drawer = new DataDrawer(g, DataDrawer.RADIUS, 2*DataDrawer.RADIUS, true);
        for(Item item: heap){
            for(int child = 2*item.pos+1, i = 0; child < heap.size() && i<2; ++child, ++i){
                Item childItem = heap.get(child);
                drawer.drawEdge(item.x, item.y + DataDrawer.RADIUS, childItem.x, childItem.y - DataDrawer.RADIUS, null);
            }
            item.drawVertex(drawer, item.x, item.y, 1);
        }
        if(last != null){
            last.drawVertex(drawer, last.x, 0, 1);
        }
        return drawer;
    }
}
