package bogqaai.graph.algorithm;

import bogqaai.graph.*;

/**
 * A Kruskal algoritmust megvalósító osztály, őse az absztrakt algoritmus
 * osztály.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class Kruskal extends Algorithm {
    /**
     * Az élek belső osztálya, a gráf éleit lehet hozzárendelni.
     */
    private static class Edge implements Comparable<Edge> {
        /** Hozzárendelt él. */
        private Graph.Edge edge;

        /**
         * A belső él osztály konstruktora, paramétere a gráf éle.
         *
         * @param edge
         */
        private Edge(Graph.Edge edge){
            this.edge = edge;
        }

        /**
         * Összehasonlító operátor az élek rendezéséhez a súlyuk szerint.
         *
         * @param o Másik él
         * @return Összehasonlítás
         */
        @Override
        public int compareTo(Edge o){
            return Integer.valueOf(edge.getWeight()).compareTo(Integer.valueOf(o.edge.getWeight()));
        }
    }

    /** Inicializáló lépés. */
    private static final String initMessage = bundle.getString("Kruskal.init.message");
    /** Sikeres lefutás eredménye. */
    private static final String resultMessage = bundle.getString("Kruskal.success.message");
    /** Lépés. */
    private static final String stepMessage = bundle.getString("Kruskal.step.message");

    /** Unió-Holvan adatszerkezet. */
    private DisjointSet set;
    /** Élek rendezett listája. */
    private final java.util.ArrayList<Edge> edges;
    /** Aktuális él. */
    private int edge = 0;

    /**
     * A Kruskal algoritmus konstruktora, paramétere a vizsgált gráf. Nagyság
     * szerint növekvő sorrendbe rendezi az éleket.
     *
     * @param graph Vizsgált gráf
     */
    public Kruskal(Graph graph){
        super(graph);
        int size = graph.getSize();
        set = new DisjointSet(graph);
        data = set;

        edges = new java.util.ArrayList<Edge>(size*size);
        java.util.List<Graph.Vertex> vertices = graph.getVertices();
        for(int i=0; i<size; ++i){
            Graph.Vertex u = vertices.get(i);
            for(int j=i+1; j<size; ++j){
                Graph.Vertex v = vertices.get(j);
                Graph.Edge e = graph.getEdge(u, v);
                if(e != null){
                    edges.add(new Edge(e));
                }
            }
        }
        java.util.Collections.sort(edges);
    }

    /**
     * Az algoritmus használja az élsúlyokat, visszatérési értéke igaz.
     *
     * @return Igaz
     */
    @Override
    public boolean isGraphWeighted() {
        return true;
    }

    /** Az algoritmus alaphelyzetbe állítása. */
    @Override
    protected String internalReset(){
        graph.resetVertices(false);
        set.fill();
        edge = 0;
        return initMessage;
    }

    /** Az algoritmus léptetése. */
    @Override
    protected String internalStep(){
        String resultText;
        if(edge < edges.size()){
            Edge e = edges.get(edge);
            Graph.Vertex u = e.edge.getVertexU();
            Graph.Vertex v = e.edge.getVertexV();
            resultText = java.text.MessageFormat.format(stepMessage, u.toString(), v.toString());
            if(set.find(u) != set.find(v)){
                e.edge.set(Graph.Color.BLUE, 0, Graph.Animation.NORMAL);
                set.union(u, v);
            }else{
                e.edge.set(Graph.Color.RED, 0, Graph.Animation.NORMAL);
            }
            ++edge;
        }else{
            result = SUCCESS;
            resultText = resultMessage;
            for(Edge e: edges){
                if(e.edge.getColor() == Graph.Color.RED){
                    e.edge.set(Graph.Color.GRAY, 0);
                }
            }
        }
        return resultText;
    }
}
