package bogqaai.graph.algorithm;

import bogqaai.graph.*;

/**
 * A magyar módszert megvalósító algoritmus osztálya, őse az absztrakt algoritmus
 * osztály.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class MaximumMatching extends Algorithm {
    /**
     * A csúcsokhoz hozzárendelt adatok az algoritmus futása során: megelőző
     * csúcs és a belőle induló párosított él.
     */
    private class Data {
        /** A csúcsból induló, párosított (fekete) él. */
        private Graph.Vertex black = null;
        /** A csúcs megelőzője a bejárás során. */
        private Graph.Vertex p = null;
    }

    /** Alternáló erdő éleinek színe. */
    private static final int COLOR = Graph.Color.BLUE;

    /** Inicializáló lépés. */
    private static final String initMessage = bundle.getString("MaximumMatching.init.message");
    /** Sikeres lefutás eredménye. */
    private static final String successMessage = bundle.getString("MaximumMatching.success.message");
    /** Sikertelen lefutás eredménye. */
    private static final String failMessage = bundle.getString("MaximumMatching.fail.message");
    /** Kezdő lépés leírása. */
    private static final String step0Message = bundle.getString("MaximumMatching.step0.message");
    /** Erdő készítés indítása lépés. */
    private static final String stepStartMessage = bundle.getString("MaximumMatching.stepstart.message");
    /** Bejárási lépés. */
    private static final String stepMessage = bundle.getString("MaximumMatching.step.message");
    /** Bejárási lépés, ha javító utat talált. */
    private static final String stepFoundMessage = bundle.getString("MaximumMatching.stepfound.message");
    /** Javító út lépése. */
    private static final String stepAlternatingMessage = bundle.getString("MaximumMatching.stepalternating.message");

    /** Kezdő lépést végző párosságot vizsgáló objektum. */
    private final Bipartite bipartite;
    /** Vizsgált gráf. */
    private final Queue queue;
    /** Javító út kezdőpontja. */
    private Graph.Vertex alternating = null;
    /** Előzőleg vizsgált csúcs. */
    private Graph.Vertex last = null;
    /** A kezdő lépés következik-e. */
    private boolean firstStep = true;
    /** Erdő készítés indítása következik-e. */
    private boolean startStep = false;

    /**
     * A magyar módszer algoritmusának konstruktora, paramétere a vizsgált gráf.
     *
     * @param graph Vizsgált gráf
     */
    public MaximumMatching(Graph graph){
        super(graph);
        bipartite = new Bipartite(graph);
        queue = new Queue(graph);
        data = queue;
    }

    /**
     * Az algoritmus nem használ élsúlyokat, visszatérési értéke hamis.
     *
     * @return Hamis
     */
    @Override
    public boolean isGraphWeighted() {
        return false;
    }

    /** Az algoritmus alaphelyzetbe állítása. */
    @Override
    protected String internalReset(){
        queue.reset();
        alternating = null;
        last = null;
        firstStep = true;
        startStep = false;
        bipartite.reset();
        return initMessage;
    }

    /** Az algoritmus léptetése. */
    @Override
    protected String internalStep(){
        String resultText;
        if(firstStep){
            // kezdőlépés: párosság vizsgálata
            firstStep = false;
            bipartite.run();
            if(bipartite.result == SUCCESS){
                resultText = step0Message;
                graph.resetEdges(true);
                for(Graph.Vertex u: graph.getVertices()){
                    u.data = new Data();
                }
                startStep = true;
            }else{
                resultText = failMessage;
                result = bipartite.result;
            }
        }else{
            if(last != null){
                last.set(last.getColor(), Graph.Fill.FILLED);
                last = null;
            }
            if(alternating != null){
                // javító út
                Graph.Vertex u = alternating;
                while(((Data)u.data).p != null){
                    Graph.Vertex v = ((Data)u.data).p;
                    ((Data)u.data).p = null;
                    int color;
                    if(v.getColor() == Bipartite.COLOR0){
                        ((Data)u.data).black = v;
                        ((Data)v.data).black = u;
                        color = Graph.Color.BLACK;
                    }else{
                        color = Graph.Color.GRAY;
                    }
                    graph.getEdge(u, v).set(color, 0, Graph.Animation.NORMAL);
                    u = v;
                }
                resultText = java.text.MessageFormat.format(stepAlternatingMessage, alternating.toString(), u.toString());
                alternating = null;
                queue.reset();
                for(Graph.Vertex v: graph.getVertices()){
                    v.set(v.getColor(), 0);
                    if(((Data)v.data).p != null){
                        Graph.Edge e = graph.getEdge(v, ((Data)v.data).p);
                        if(e.getColor() == COLOR){
                            e.set(Graph.Color.GRAY, 0);
                        }
                        ((Data)v.data).p = null;
                    }
                }
                startStep = true;
            }else if(startStep){
                // erdő készítés indítása
                startStep = false;
                resultText = stepStartMessage;
                for(Graph.Vertex u: graph.getVertices()){
                    if(u.getColor() == Bipartite.COLOR0 && ((Data)u.data).black == null){
                        u.set(u.getColor(), Graph.Fill.FILLED, Graph.Animation.NORMAL);
                        queue.push(u);
                    }
                }
            }else if(!queue.isEmpty()){
                // bejárási lépés
                Graph.Vertex u = queue.pop();
                resultText = java.text.MessageFormat.format(stepMessage, u.toString());
                if(u.getColor() == Bipartite.COLOR0){
                    for(Graph.Vertex v: graph.getVertices()){
                        Graph.Edge e = graph.getEdge(u, v);
                        if(e != null && ((Data)v.data).p == null){
                            v.set(v.getColor(), Graph.Fill.FILLED, Graph.Animation.DELAYED);
                            e.set(COLOR, 0, e.getVertexU() == u ? Graph.Animation.LTR : Graph.Animation.RTL);
                            ((Data)v.data).p = u;
                            if(((Data)v.data).black == null){
                                resultText = java.text.MessageFormat.format(stepFoundMessage, u.toString(), v.toString());
                                alternating = v;
                                break;
                            }else{
                                queue.push(v);
                            }
                        }
                    }
                }else{
                    Graph.Vertex v = ((Data)u.data).black;
                    v.set(v.getColor(), Graph.Fill.FILLED, Graph.Animation.NORMAL);
                    ((Data)v.data).p = u;
                    queue.push(v);
                }
                u.set(u.getColor(), Graph.Fill.MARKED);
                last = u;
            }else{
                // kész
                resultText = successMessage;
                result = SUCCESS;
                queue.reset();
                for(Graph.Vertex u: graph.getVertices()){
                    u.set(u.getColor(), 0);
                    if(((Data)u.data).p != null){
                        Graph.Edge e = graph.getEdge(u, ((Data)u.data).p);
                        if(e.getColor() == COLOR){
                            e.set(Graph.Color.GRAY, 0);
                        }
                    }
                    u.data = null;
                }
            }
        }
        return resultText;
    }
}
