package bogqaai.graph.algorithm;

import bogqaai.graph.*;

/**
 * A sor adatszerkezetet megvalósító osztály a gráf csúcsainak tárolására, az
 * absztrakt adatszerkezet osztály leszármazottja. A sor rendelkezik a szokásos
 * műveletekkel - elem berakása, kivétele, üresség lekérdezése -, kirajzolásnál
 * megjeleníti a sorból legutoljára kivett elemet és még a sorban lévő elemeket.
 * A sort egy láncolt lista segítségével implementálja, a sor műveleteit a lista
 * műveletein keresztül valósítja meg.
 *
 * @author Bognár Gergő, BOGQAAI.ELTE
 */
public final class Queue extends AbstractData {
    /**
     * A sor elemeinek belső osztálya, az absztrakt gráf elem osztály
     * leszármazottja. Az elemekhez a gráf csúcsait lehet hozzárendelni.
     */
    private class Item extends GraphElement{
        /** Hozzárendelt csúcs. */
        private final Graph.Vertex vertex;

        /**
         * A sor elem osztály konstruktora, paramétere a hozzárendelt csúcs.
         *
         * @param vertex Hozzárendelt csúcs
         */
        private Item(Graph.Vertex vertex){
            this.vertex = vertex;
        }

        /**
         * Az elem felirata a hozzárendelt csúcs felirata.
         *
         * @return Felirat
         */
        @Override
        public String toString(){
            return vertex.toString();
        }
    }

    /** Sort implementáló láncolt lista. */
    private final java.util.LinkedList<Item> queue = new java.util.LinkedList<Item>();
    /** A sorból utoljára kivett elem. */
    private Item last = null;

    /**
     * A sor adatszerkezetet konstruktora, paramétere a vizsgált gráf.
     *
     * @param graph Vizsgált gráf
     */
    public Queue(Graph graph){
        int size = graph.getSize();
        drawerWidth = size > 0 ? (size-1)*DataDrawer.DISTANCE : 0;
        drawerHeight = 2*DataDrawer.DISTANCE;
    }

    /**
     * Csúcs berakása a sor végére.
     *
     * @param u Berakandó csúcs
     * @return Sikeresség
     */
    public boolean push(Graph.Vertex u){
        Item item = new Item(u);
        item.set(AbstractGraph.Color.WHITE, 0);
        item.set(0, 0, AbstractGraph.Animation.NORMAL);
        return queue.add(new Item(u));
    }
    /**
     * Legelső csúcs kivétele a sorból.
     *
     * @return Legelső csúcs
     */
    public Graph.Vertex pop(){
        last = queue.remove();
        last.set(0, Graph.Fill.MARKED);
        return last.vertex;
    }
    /**
     * A sor ürességének vizsgálata. Visszatérési értéke igaz, ha a sor üres.
     *
     * @return Üres-e
     */
    public boolean isEmpty(){
        return queue.isEmpty();
    }
    /**
     * Sor alaphelyzetbe állítása, a sor elemeinek és a sorból utoljára kivett
     * elemnek a törlése.
     */
    public void reset(){
        queue.clear();
        last = null;
    }

    /**
     * Adatszerkezet beállításainak törlése.
     */
    @Override
    protected void internalClear(){
        reset();
    }

    /**
     * Sor és a sorból legutoljára kivett elem kirajzolása. Visszatérési értéke
     * a rajzoló objektum.
     *
     * @param g A rajzolási cél
     * @return A rajzoló objektum
     */
    @Override
    public Drawer draw(java.awt.Graphics2D g){
        Drawer drawer = new DataDrawer(g);
        if(queue.size() > 0){
            drawer.drawEdge(0, DataDrawer.DISTANCE, (queue.size()-1)*DataDrawer.DISTANCE, DataDrawer.DISTANCE, null);
        }
        for(int i=0; i<queue.size(); ++i){
            Item item = queue.get(i);
            item.drawVertex(drawer, i*DataDrawer.DISTANCE, DataDrawer.DISTANCE);
        }
        if(last != null){
            last.drawVertex(drawer, 0, 0, 1);
        }
        return drawer;
    }
}
